@@ -1,4 +1,4 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
 # Copyright (C) 2003-2010 Edgewall Software
@@ -12,7 +12,7 @@
 # individuals. For the exact contribution history, see the revision
 # history and logs, available at http://trac.edgewall.org/log/.
 
-from __future__ import with_statement
+from __future__ import print_function
 
 import cmd
 import os.path
@@ -25,6 +25,7 @@ import traceback
 from trac import __version__ as VERSION
 from trac.admin.api import AdminCommandError, AdminCommandManager, \
                            get_console_locale
+from trac.config import Configuration
 from trac.core import TracError
 from trac.env import Environment
 from trac.ticket.model import *
@@ -35,6 +36,7 @@ from trac.util.text import console_print, exception_to_unicode, printout, \
                            getpreferredencoding
 from trac.util.translation import _, ngettext, has_babel, cleandoc_
 from trac.versioncontrol.api import RepositoryManager
+from trac.web.chrome import default_mainnav_order, default_metanav_order
 from trac.wiki.admin import WikiAdmin
 from trac.wiki.formatter import MacroError
 from trac.wiki.macros import WikiMacroBase
@@ -110,16 +112,16 @@ class TracAdmin(cmd.Cmd):
             rv = cmd.Cmd.onecmd(self, line) or 0
         except SystemExit:
             raise
-        except AdminCommandError, e:
+        except AdminCommandError as e:
             printerr(_("Error: %(msg)s", msg=to_unicode(e)))
             if e.show_usage:
-                print
+                print()
                 self.do_help(e.cmd or self.arg_tokenize(line)[0])
             rv = 2
-        except TracError, e:
+        except TracError as e:
             printerr(exception_to_unicode(e))
             rv = 2
-        except Exception, e:
+        except Exception as e:
             printerr(exception_to_unicode(e))
             rv = 2
             if self.env_check():
@@ -163,7 +165,7 @@ Type:  '?' or 'help' for help on commands.
             if not self.__env:
                 self._init_env()
             return self.__env
-        except Exception, e:
+        except Exception as e:
             printerr(_("Failed to open environment: %(err)s",
                        err=exception_to_unicode(e, traceback=True)))
             sys.exit(1)
@@ -248,7 +250,7 @@ Type:  '?' or 'help' for help on commands.
         if self.env_check():
             try:
                 comp = self.cmd_mgr.complete_command(args, cmd_only)
-            except Exception, e:
+            except Exception as e:
                 printerr()
                 printerr(_('Completion error: %(err)s',
                            err=exception_to_unicode(e)))
@@ -275,16 +277,16 @@ Type:  '?' or 'help' for help on commands.
                 self._init_env()
             if self.needs_upgrade is None:
                 self.needs_upgrade = self.__env.needs_upgrade()
-        except TracError, e:
+        except TracError as e:
             raise AdminCommandError(to_unicode(e))
-        except Exception, e:
+        except Exception as e:
             raise AdminCommandError(exception_to_unicode(e))
         args = self.arg_tokenize(line)
         if args[0] == 'upgrade':
             self.needs_upgrade = None
         elif self.needs_upgrade:
-            raise TracError(_('The Trac Environment needs to be upgraded.\n\n'
-                              'Run "trac-admin %(path)s upgrade"',
+            raise TracError(_('The Trac Environment needs to be upgraded. '
+                              'Run:\n\n  trac-admin "%(path)s" upgrade',
                               path=self.envname))
         return self.cmd_mgr.execute_command(*args)
 
@@ -333,7 +335,7 @@ Type:  '?' or 'help' for help on commands.
             printout(_("trac-admin - The Trac Administration Console "
                        "%(version)s", version=TRAC_VERSION))
             if not self.interactive:
-                print
+                print()
                 printout(_("Usage: trac-admin </path/to/projenv> "
                            "[command [subcommand] [option ...]]\n")
                     )
@@ -349,7 +351,7 @@ Type:  '?' or 'help' for help on commands.
     _help_EOF = _help_quit
 
     def do_quit(self, line):
-        print
+        print()
         sys.exit()
 
     do_exit = do_quit # Alias
@@ -370,6 +372,12 @@ Type:  '?' or 'help' for help on commands.
          files are written to the conf/trac.ini file of the newly created
          environment. Relative paths are resolved relative to the "conf"
          directory of the new environment.
+
+         The optional argument --config=PATH can be used to specify a
+         configuration file that is used to populate the environment
+         configuration. The arguments <projectname>, <db> and any other
+         arguments passed in the invocation will override values in the
+         configuration file.
          """)]
 
     def do_initdb(self, line):
@@ -399,13 +407,13 @@ in order to initialize and prepare the project database.
         ddb = 'sqlite:db/trac.db'
         prompt = _("Database connection string [%(default)s]> ", default=ddb)
         returnvals.append(raw_input(prompt).strip() or ddb)
-        print
+        print()
         return returnvals
 
     def do_initenv(self, line):
         def initenv_error(msg):
             printerr(_("Initenv for '%(env)s' failed.", env=self.envname),
-                     "\n" + msg)
+                     "\n%s" % msg)
         if self.env_check():
             initenv_error(_("Does an environment already exist?"))
             return 2
@@ -422,13 +430,28 @@ in order to initialize and prepare the project database.
 
         arg = self.arg_tokenize(line)
         inherit_paths = []
+        config_file_path = None
         i = 0
         while i < len(arg):
             item = arg[i]
             if item.startswith('--inherit='):
                 inherit_paths.append(arg.pop(i)[10:])
+            elif item.startswith('--config='):
+                config_file_path = arg.pop(i)[9:]
             else:
                 i += 1
+        config = None
+        if config_file_path:
+            if not os.path.exists(config_file_path):
+                initenv_error(_("The file specified in the --config argument "
+                                "does not exist: %(path)s.",
+                                path=config_file_path))
+                return 2
+            try:
+                config = Configuration(config_file_path)
+            except TracError as e:
+                initenv_error(e)
+                return 2
         arg = arg or [''] # Reset to usual empty in case we popped the only one
         project_name = None
         db_str = None
@@ -446,14 +469,25 @@ in order to initialize and prepare the project database.
 
         try:
             printout(_("Creating and Initializing Project"))
-            options = [
+            options = []
+            if config:
+                for section in config.sections(defaults=False):
+                    options.extend((section, option, value)
+                                   for option, value
+                                   in config.options(section))
+            options.extend([
                 ('project', 'name', project_name),
                 ('trac', 'database', db_str),
-            ]
+            ])
+            def add_nav_order_options(section, default):
+                for i, name in enumerate(default, 1):
+                    options.append((section, name + '.order', float(i)))
+            add_nav_order_options('mainnav', default_mainnav_order)
+            add_nav_order_options('metanav', default_metanav_order)
             if repository_dir:
                 options.extend([
-                    ('trac', 'repository_type', repository_type),
-                    ('trac', 'repository_dir', repository_dir),
+                    ('repositories', '.type', repository_type),
+                    ('repositories', '.dir', repository_dir),
                 ])
             if inherit_paths:
                 options.append(('inherit', 'file',
@@ -461,7 +495,7 @@ in order to initialize and prepare the project database.
             try:
                 self.__env = Environment(self.envname, create=True,
                                          options=options)
-            except Exception, e:
+            except Exception as e:
                 initenv_error(_('Failed to create environment.'))
                 printerr(e)
                 traceback.print_exc()
@@ -479,7 +513,7 @@ in order to initialize and prepare the project database.
                     if repos:
                         printout(_(" Indexing default repository"))
                         repos.sync(self._resync_feedback)
-                except TracError, e:
+                except TracError as e:
                     printerr(_("""
 ---------------------------------------------------------------------
 Warning: couldn't index the default repository.
@@ -492,7 +526,7 @@ You can nevertheless start using your Trac environment, but
 you'll need to check again your trac.ini file and the [trac]
 repository_type and repository_path settings.
 """))
-        except Exception, e:
+        except Exception as e:
             initenv_error(to_unicode(e))
             traceback.print_exc()
             return 2
@@ -573,8 +607,11 @@ def _run(args):
     if args is None:
         args = sys.argv[1:]
     warn_setuptools_issue()
+    if sys.flags.optimize != 0:
+        printerr(_("Python with optimizations is not supported."))
+        return 2
     admin = TracAdmin()
-    if len(args) > 0:
+    if args:
         if args[0] in ('-h', '--help', 'help'):
             return admin.onecmd(' '.join(_quote_args(['help'] + args[1:])))
         elif args[0] in ('-v','--version'):
