@@ -14,6 +14,7 @@
 #
 # Author: Christopher Lenz <cmlenz@gmx.de>
 
+import errno
 import os
 import re
 import weakref
@@ -22,9 +23,11 @@ from genshi.builder import tag
 
 from trac.config import ConfigurationError, ListOption
 from trac.core import Component, TracError, implements
-from trac.db.api import IDatabaseConnector
+from trac.db.api import ConnectionBase, IDatabaseConnector
+from trac.db.schema import Table, Column, Index
 from trac.db.util import ConnectionWrapper, IterableCursor
-from trac.util import get_pkginfo, getuser
+from trac.env import ISystemInfoProvider
+from trac.util import get_pkginfo, getuser, lazy
 from trac.util.translation import _, tag_
 
 _like_escape_re = re.compile(r'([/_%])')
@@ -33,77 +36,81 @@ _glob_escape_re = re.compile(r'[*?\[]')
 
 try:
     import pysqlite2.dbapi2 as sqlite
-    have_pysqlite = 2
 except ImportError:
-    try:
-        import sqlite3 as sqlite
-        have_pysqlite = 2
-    except ImportError:
-        have_pysqlite = 0
+    import sqlite3 as sqlite
 
-if have_pysqlite == 2:
-    # Force values to integers because PySQLite 2.2.0 had (2, 2, '0')
-    sqlite_version = tuple([int(x) for x in sqlite.sqlite_version_info])
-    sqlite_version_string = sqlite.sqlite_version
+sqlite_version = sqlite.sqlite_version_info
+sqlite_version_string = sqlite.sqlite_version
+pysqlite_version = sqlite.version_info
+pysqlite_version_string = get_pkginfo(sqlite).get('version',
+                                                  '%d.%d.%s'
+                                                  % pysqlite_version)
+min_sqlite_version = (3, 0, 0)
+min_pysqlite_version = (2, 4, 1)  # version provided by Python 2.6
 
-    class PyFormatCursor(sqlite.Cursor):
-        def _rollback_on_error(self, function, *args, **kwargs):
-            try:
-                return function(self, *args, **kwargs)
-            except sqlite.DatabaseError:
-                self.cnx.rollback()
-                raise
-        def execute(self, sql, args=None):
-            if args:
-                sql = sql % (('?',) * len(args))
-            return self._rollback_on_error(sqlite.Cursor.execute, sql,
-                                           args or [])
-        def executemany(self, sql, args):
-            if not args:
-                return
-            sql = sql % (('?',) * len(args[0]))
-            return self._rollback_on_error(sqlite.Cursor.executemany, sql,
-                                           args)
 
-    # EagerCursor taken from the example in pysqlite's repository:
-    #
-    #   http://code.google.com/p/pysqlite/source/browse/misc/eager.py
-    #
-    # Only change is to subclass it from PyFormatCursor instead of
-    # sqlite.Cursor.
+class PyFormatCursor(sqlite.Cursor):
+    def _rollback_on_error(self, function, *args, **kwargs):
+        try:
+            return function(self, *args, **kwargs)
+        except sqlite.DatabaseError:
+            self.cnx.rollback()
+            raise
 
-    class EagerCursor(PyFormatCursor):
-        def __init__(self, con):
-            PyFormatCursor.__init__(self, con)
-            self.rows = []
-            self.pos = 0
+    def execute(self, sql, args=None):
+        if args:
+            sql = sql % (('?',) * len(args))
+        return self._rollback_on_error(sqlite.Cursor.execute, sql,
+                                       args or [])
 
-        def execute(self, *args):
-            result = PyFormatCursor.execute(self, *args)
-            self.rows = PyFormatCursor.fetchall(self)
-            self.pos = 0
-            return result
+    def executemany(self, sql, args):
+        if not args:
+            return
+        sql = sql % (('?',) * len(args[0]))
+        return self._rollback_on_error(sqlite.Cursor.executemany, sql,
+                                       args)
 
-        def fetchone(self):
-            try:
-                row = self.rows[self.pos]
-                self.pos += 1
-                return row
-            except IndexError:
-                return None
 
-        def fetchmany(self, num=None):
-            if num is None:
-                num = self.arraysize
+# EagerCursor taken from the example in pysqlite's repository:
+#
+#   http://code.google.com/p/pysqlite/source/browse/misc/eager.py
+#
+# Only change is to subclass it from PyFormatCursor instead of
+# sqlite.Cursor.
 
-            result = self.rows[self.pos:self.pos+num]
-            self.pos += num
-            return result
+class EagerCursor(PyFormatCursor):
+    def __init__(self, con):
+        PyFormatCursor.__init__(self, con)
+        self.rows = []
+        self.pos = 0
 
-        def fetchall(self):
-            result = self.rows[self.pos:]
-            self.pos = len(self.rows)
-            return result
+    def execute(self, *args):
+        result = PyFormatCursor.execute(self, *args)
+        self.rows = PyFormatCursor.fetchall(self)
+        self.pos = 0
+        return result
+
+    def fetchone(self):
+        try:
+            row = self.rows[self.pos]
+        except IndexError:
+            return None
+        else:
+            self.pos += 1
+            return row
+
+    def fetchmany(self, num=None):
+        if num is None:
+            num = self.arraysize
+
+        result = self.rows[self.pos:self.pos + num]
+        self.pos += num
+        return result
+
+    def fetchall(self):
+        result = self.rows[self.pos:]
+        self.pos = len(self.rows)
+        return result
 
 
 # Mapping from "abstract" SQL types to DB-specific types
@@ -142,48 +149,52 @@ class SQLiteConnector(Component):
     sqlite:path/to/trac.db
     }}}
     """
-    implements(IDatabaseConnector)
+    implements(IDatabaseConnector, ISystemInfoProvider)
+
+    required = False
 
     extensions = ListOption('sqlite', 'extensions',
-        doc="""Paths to sqlite extensions, relative to Trac environment's
-        directory or absolute. (''since 0.12'')""")
+        doc="""Paths to [https://sqlite.org/loadext.html sqlite extensions].
+        The paths may be absolute or relative to the Trac environment.
+        (''since 0.12'')
+        """)
 
     memory_cnx = None
 
     def __init__(self):
-        self._version = None
         self.error = None
-        self._extensions = None
+
+    # ISystemInfoProvider methods
+
+    def get_system_info(self):
+        if self.required:
+            yield 'SQLite', sqlite_version_string
+            yield 'pysqlite', pysqlite_version_string
+
+    # IDatabaseConnector methods
 
     def get_supported_schemes(self):
-        if not have_pysqlite:
-            self.error = _("Cannot load Python bindings for SQLite")
-        elif sqlite_version >= (3, 3, 3) and sqlite.version_info[0] == 2 and \
-                sqlite.version_info < (2, 0, 7):
+        if sqlite_version < min_sqlite_version:
+            self.error = _("SQLite version is %(version)s. Minimum required "
+                           "version is %(min_version)s.",
+                           version=sqlite_version_string,
+                           min_version='%d.%d.%d' % min_sqlite_version)
+        elif pysqlite_version < min_pysqlite_version:
             self.error = _("Need at least PySqlite %(version)s or higher",
-                           version='2.0.7')
-        elif (2, 5, 2) <= sqlite.version_info < (2, 5, 5):
+                           version='%d.%d.%d' % min_pysqlite_version)
+        elif (2, 5, 2) <= pysqlite_version < (2, 5, 5):
             self.error = _("PySqlite 2.5.2 - 2.5.4 break Trac, please use "
                            "2.5.5 or higher")
-        yield ('sqlite', -1 if self.error else 1)
+        yield 'sqlite', -1 if self.error else 1
 
     def get_connection(self, path, log=None, params={}):
-        if not self._version:
-            self._version = get_pkginfo(sqlite).get(
-                'version', '%d.%d.%s' % sqlite.version_info)
-            self.env.systeminfo.extend([('SQLite', sqlite_version_string),
-                                        ('pysqlite', self._version)])
-            self.required = True
-        # construct list of sqlite extension libraries
-        if self._extensions is None:
-            self._extensions = []
-            for extpath in self.extensions:
-                if not os.path.isabs(extpath):
-                    extpath = os.path.join(self.env.path, extpath)
-                self._extensions.append(extpath)
+        self.required = True
         params['extensions'] = self._extensions
         if path == ':memory:':
-            if not self.memory_cnx:
+            try:
+                self.memory_cnx.cursor()
+            except (AttributeError, sqlite.DatabaseError):
+                # memory_cnx is None or database connection closed.
                 self.memory_cnx = SQLiteConnection(path, log, params)
             return self.memory_cnx
         else:
@@ -195,13 +206,13 @@ class SQLiteConnector(Component):
     def init_db(self, path, schema=None, log=None, params={}):
         if path != ':memory:':
             # make the directory to hold the database
-            if os.path.exists(path):
+            if self.db_exists(path):
                 raise TracError(_("Database already exists at %(path)s",
                                   path=path))
             dir = os.path.dirname(path)
             if not os.path.exists(dir):
                 os.makedirs(dir)
-            if isinstance(path, unicode): # needed with 2.4.0
+            if isinstance(path, unicode):  # needed with 2.4.0
                 path = path.encode('utf-8')
             # this direct connect will create the database if needed
             cnx = sqlite.connect(path, isolation_level=None,
@@ -221,6 +232,19 @@ class SQLiteConnector(Component):
         cursor.close()
         cnx.commit()
 
+    def destroy_db(self, path, log=None, params={}):
+        if path != ':memory:':
+            if not os.path.isabs(path):
+                path = os.path.join(self.env.path, path)
+            try:
+                os.remove(path)
+            except OSError as e:
+                if e.errno != errno.ENOENT:
+                    raise
+
+    def db_exists(self, path, log=None, params={}):
+        return os.path.exists(path)
+
     def to_sql(self, table):
         return _to_sql(table)
 
@@ -233,14 +257,14 @@ class SQLiteConnector(Component):
         """
         for name, (from_, to) in sorted(columns.iteritems()):
             if _type_map.get(to, to) != _type_map.get(from_, from_):
-                raise NotImplementedError('Conversion from %s to %s is not '
-                                          'implemented' % (from_, to))
+                raise NotImplementedError("Conversion from %s to %s is not "
+                                          "implemented" % (from_, to))
         return ()
 
     def backup(self, dest_file):
         """Simple SQLite-specific backup of the database.
 
-        @param dest_file: Destination file basename
+        :param dest_file: Destination file basename
         """
         import shutil
         db_str = self.config.get('trac', 'database')
@@ -254,18 +278,24 @@ class SQLiteConnector(Component):
             raise TracError(_("No destination file created"))
         return dest_file
 
+    @lazy
+    def _extensions(self):
+        _extensions = []
+        for extpath in self.extensions:
+            if not os.path.isabs(extpath):
+                extpath = os.path.join(self.env.path, extpath)
+            _extensions.append(extpath)
+        return _extensions
 
-class SQLiteConnection(ConnectionWrapper):
+
+class SQLiteConnection(ConnectionBase, ConnectionWrapper):
     """Connection wrapper for SQLite."""
 
     __slots__ = ['_active_cursors', '_eager']
 
-    poolable = have_pysqlite and sqlite_version >= (3, 3, 8) \
-                             and sqlite.version_info >= (2, 5, 0)
+    poolable = sqlite_version >= (3, 3, 8) and pysqlite_version >= (2, 5, 0)
 
     def __init__(self, path, log=None, params={}):
-        if have_pysqlite == 0:
-            raise TracError(_("Cannot load Python bindings for SQLite"))
         self.cnx = None
         if path != ':memory:':
             if not os.access(path, os.F_OK):
@@ -274,17 +304,18 @@ class SQLiteConnection(ConnectionWrapper):
 
             dbdir = os.path.dirname(path)
             if not os.access(path, os.R_OK + os.W_OK) or \
-                   not os.access(dbdir, os.R_OK + os.W_OK):
+                    not os.access(dbdir, os.R_OK + os.W_OK):
                 raise ConfigurationError(tag_(
                     "The user %(user)s requires read _and_ write permissions "
                     "to the database file %(path)s and the directory it is "
-                    "located in.", user=tag.tt(getuser()), path=tag.tt(path)))
+                    "located in.", user=tag.code(getuser()),
+                    path=tag.code(path)))
 
         self._active_cursors = weakref.WeakKeyDictionary()
         timeout = int(params.get('timeout', 10.0))
         self._eager = params.get('cursor', 'eager') == 'eager'
         # eager is default, can be turned off by specifying ?cursor=
-        if isinstance(path, unicode): # needed with 2.4.0
+        if isinstance(path, unicode):  # needed with 2.4.0
             path = path.encode('utf-8')
         cnx = sqlite.connect(path, detect_types=sqlite.PARSE_DECLTYPES,
                              isolation_level=None,
@@ -328,6 +359,29 @@ class SQLiteConnection(ConnectionWrapper):
     def concat(self, *args):
         return '||'.join(args)
 
+    def drop_column(self, table, column):
+        column_names = self.get_column_names(table)
+        if column in column_names:
+            table_schema = self._get_table_schema(table)
+            table_schema.remove_columns([column])
+            temp_table = table + '_old'
+            table_name = self.quote(table)
+            temp_table_name = self.quote(temp_table)
+            column_names.remove(column)
+            cols_to_copy = ','.join(self.quote(col) for col in column_names)
+            cursor = self.cursor()
+            cursor.execute("""
+                CREATE TEMPORARY TABLE %s AS SELECT * FROM %s
+                """ % (temp_table_name, table_name))
+            self.drop_table(table)
+            for sql in _to_sql(table_schema):
+                cursor.execute(sql)
+            cursor.execute("""
+                INSERT INTO %s (%s) SELECT %s FROM %s
+                """ % (table_name, cols_to_copy, cols_to_copy,
+                       temp_table_name))
+            self.drop_table(temp_table)
+
     def drop_table(self, table):
         cursor = self.cursor()
         if sqlite_version < (3, 7, 6):
@@ -341,21 +395,20 @@ class SQLiteConnection(ConnectionWrapper):
             cursor.execute("DROP TABLE IF EXISTS " + self.quote(table))
 
     def get_column_names(self, table):
-        cursor = self.cnx.cursor()
-        rows = cursor.execute("PRAGMA table_info(%s)"
-                              % self.quote(table))
-        return [row[1] for row in rows]
+        cursor = self.cursor()
+        cursor.execute("PRAGMA table_info(%s)" % self.quote(table))
+        return [row[1] for row in cursor]
 
     def get_last_id(self, cursor, table, column='id'):
         return cursor.lastrowid
 
     def get_table_names(self):
         rows = self.execute("""
-            SELECT name FROM sqlite_master WHERE type='table'""")
+            SELECT name FROM sqlite_master WHERE type='table'
+            """)
         return [row[0] for row in rows]
 
     def like(self):
-        """Return a case-insensitive LIKE clause."""
         if sqlite_version >= (3, 1, 0):
             return "LIKE %s ESCAPE '/'"
         else:
@@ -368,22 +421,54 @@ class SQLiteConnection(ConnectionWrapper):
             return text
 
     def prefix_match(self):
-        """Return a case sensitive prefix-matching operator."""
         return 'GLOB %s'
 
     def prefix_match_value(self, prefix):
-        """Return a value for case sensitive prefix-matching operator."""
         return _glob_escape_re.sub(lambda m: '[%s]' % m.group(0), prefix) + '*'
 
     def quote(self, identifier):
-        """Return the quoted identifier."""
         return _quote(identifier)
 
+    def reset_tables(self):
+        cursor = self.cursor()
+        table_names = self.get_table_names()
+        for name in table_names:
+            cursor.execute("DELETE FROM %s" % name)
+        return table_names
+
     def update_sequence(self, cursor, table, column='id'):
         # SQLite handles sequence updates automagically
         # http://www.sqlite.org/autoinc.html
         pass
 
+    def _get_table_schema(self, table):
+        key = None
+        items = []
+        cursor = self.cursor()
+        cursor.execute("PRAGMA table_info(%s)" % self.quote(table))
+        for row in cursor:
+            column = row[1]
+            type_ = row[2]
+            pk = row[5]
+            if pk == 1 and type_ == 'integer':
+                key = [column]
+                auto_increment = True
+            else:
+                auto_increment = False
+            items.append(Column(column, type=type_,
+                                auto_increment=auto_increment))
+        cursor.execute("PRAGMA index_list(%s)" % self.quote(table))
+        for row in cursor.fetchall():
+            index = row[1]
+            unique = row[2]
+            cursor.execute("PRAGMA index_info(%s)" % self.quote(index))
+            columns = [row[2] for row in cursor]
+            if key is None and index.startswith('sqlite_autoindex_'):
+                key = columns
+            else:
+                items.append(Index(columns, unique=bool(unique)))
+        return Table(table, key=key or [])[items]
+
 
 def _quote(identifier):
     return "`%s`" % identifier.replace('`', '``')
@@ -394,16 +479,16 @@ def _set_journal_mode(cursor, value):
         return
     value = value.upper()
     if value == 'OFF':
-        raise TracError(_("PRAGMA journal_mode `%(value)s` cannot be used in "
-                          "SQLite", value=value))
+        raise TracError(_("PRAGMA journal_mode `%(value)s` cannot be used "
+                          "in SQLite", value=value))
     cursor.execute('PRAGMA journal_mode = %s' % _quote(value))
     row = cursor.fetchone()
     if not row:
         raise TracError(_("PRAGMA journal_mode isn't supported by SQLite "
                           "%(version)s", version=sqlite_version_string))
     if (row[0] or '').upper() != value:
-        raise TracError(_("PRAGMA journal_mode `%(value)s` isn't supported by "
-                          "SQLite %(version)s",
+        raise TracError(_("PRAGMA journal_mode `%(value)s` isn't supported "
+                          "by SQLite %(version)s",
                           value=value, version=sqlite_version_string))
 
 
