@@ -16,10 +16,11 @@
 # Author: Jonas Borgström <jonas@edgewall.com>
 #         Christopher Lenz <cmlenz@gmx.de>
 
-from __future__ import with_statement
+from __future__ import print_function
 
 import csv
 import os
+from itertools import groupby
 
 from trac.admin import AdminCommandError, IAdminCommandProvider, get_dir_list
 from trac.cache import cached
@@ -36,7 +37,7 @@ __all__ = ['IPermissionRequestor', 'IPermissionStore', 'IPermissionPolicy',
            'IPermissionGroupProvider', 'PermissionError', 'PermissionSystem']
 
 
-class PermissionError(StandardError):
+class PermissionError(StandardError, TracBaseError):
     """Insufficient permissions to perform the operation.
 
     :since 1.0.5: the `msg` attribute is deprecated and will be removed in
@@ -229,7 +230,6 @@ class DefaultPermissionStore(Component):
         formatted tuples."""
         return self._all_permissions
 
-
     @cached
     def _all_permissions(self):
         return [(username, action) for username, action in
@@ -305,7 +305,6 @@ class DefaultPermissionPolicy(Component):
         return action in permissions or None
 
 
-
 class PermissionSystem(Component):
     """Permission management sub-system."""
 
@@ -322,14 +321,11 @@ class PermissionSystem(Component):
 
     policies = OrderedExtensionsOption('trac', 'permission_policies',
         IPermissionPolicy,
-        'DefaultPermissionPolicy, LegacyAttachmentPolicy',
+        'ReadonlyWikiPolicy, DefaultPermissionPolicy, LegacyAttachmentPolicy',
         False,
-        """List of components implementing `IPermissionPolicy`, in the order in
-        which they will be applied. These components manage fine-grained access
-        control to Trac resources.
-        Defaults to the DefaultPermissionPolicy (pre-0.11 behavior) and
-        LegacyAttachmentPolicy (map ATTACHMENT_* permissions to realm specific
-        ones)""")
+        """List of components implementing `IPermissionPolicy`, in the order
+        in which they will be applied. These components manage fine-grained
+        access control to Trac resources.""")
 
     # Number of seconds a cached user permission set is valid for.
     CACHE_EXPIRY = 5
@@ -343,15 +339,20 @@ class PermissionSystem(Component):
     # Public API
 
     def grant_permission(self, username, action):
-        """Grant the user with the given name permission to perform to specified
-        action."""
+        """Grant the user with the given name permission to perform to
+        specified action."""
         if action.isupper() and action not in self.get_actions():
             raise TracError(_('%(name)s is not a valid action.', name=action))
+        elif not action.isupper() and action.upper() in self.get_actions():
+            raise TracError(_("Permission %(name)s differs from a defined "
+                              "action by casing only, which is not allowed.",
+                              name=action))
 
         self.store.grant_permission(username, action)
 
     def revoke_permission(self, username, action):
-        """Revokes the permission of the specified user to perform an action."""
+        """Revokes the permission of the specified user to perform an
+        action."""
         self.store.revoke_permission(username, action)
 
     def get_actions_dict(self):
@@ -383,6 +384,33 @@ class PermissionSystem(Component):
                     actions.add(action)
         return list(actions)
 
+    def get_groups_dict(self):
+        """Get all groups as a `dict`.
+
+        The keys are the group names. The values are the group members.
+
+        :since: 1.1.3
+        """
+        groups = sorted((p for p in self.get_all_permissions()
+                           if not p[1].isupper()), key=lambda p: p[1])
+
+        return dict((k, sorted(i[0] for i in list(g)))
+                    for k, g in groupby(groups, key=lambda p: p[1]))
+
+    def get_users_dict(self):
+        """Get all users as a `dict`.
+
+        The keys are the user names. The values are the actions possessed
+        by the user.
+
+        :since: 1.1.3
+        """
+        perms = sorted((p for p in self.get_all_permissions()
+                          if p[1].isupper()), key=lambda p: p[0])
+
+        return dict((k, sorted(i[1] for i in list(g)))
+                    for k, g in groupby(perms, key=lambda p: p[0]))
+
     def get_user_permissions(self, username=None):
         """Return the permissions of the specified user.
 
@@ -456,7 +484,8 @@ class PermissionSystem(Component):
             expand_action(a)
         return expanded_actions
 
-    def check_permission(self, action, username=None, resource=None, perm=None):
+    def check_permission(self, action, username=None, resource=None,
+                         perm=None):
         """Return True if permission to perform action for the given resource
         is allowed."""
         if username is None:
@@ -480,14 +509,9 @@ class PermissionSystem(Component):
 
     def get_permission_actions(self):
         """Implement the global `TRAC_ADMIN` meta permission.
-
-        Implements also the `EMAIL_VIEW` permission which allows for
-        showing email addresses even if `[trac] show_email_addresses`
-        is `false`.
         """
         actions = self.get_actions(skip=self)
-        actions.append('EMAIL_VIEW')
-        return [('TRAC_ADMIN', actions), 'EMAIL_VIEW']
+        return [('TRAC_ADMIN', actions)]
 
 
 class PermissionCache(object):
@@ -549,7 +573,8 @@ class PermissionCache(object):
             'WIKI_VIEW' in perm(resource)
 
         """
-        resource = Resource(realm_or_resource, id, version)
+        resource = Resource(realm_or_resource, id, version) \
+                   if realm_or_resource else None
         if resource and self._resource and resource == self._resource:
             return self
         else:
@@ -568,6 +593,8 @@ class PermissionCache(object):
             cache_decision, cache_resource = cached
             if resource == cache_resource:
                 return cache_decision
+        # Avoid recursion in policies that call has_permission.
+        self._cache[key] = (False, resource)
         perm = self
         if resource is not self._resource:
             perm = PermissionCache(self.env, self.username, resource,
@@ -589,14 +616,6 @@ class PermissionCache(object):
                 raise PermissionError(msg=message)
     assert_permission = require
 
-    def permissions(self):
-        """Deprecated (but still used by the HDF compatibility layer)"""
-        self.env.log.warning("perm.permissions() is deprecated and "
-                             "is only present for HDF compatibility")
-        perm = PermissionSystem(self.env)
-        actions = perm.get_user_permissions(self.username)
-        return [action for action in actions if action in self]
-
 
 class PermissionAdmin(Component):
     """trac-admin command provider for permission system administration."""
@@ -664,14 +683,14 @@ class PermissionAdmin(Component):
             rows = permsys.get_all_permissions()
         rows.sort()
         print_table(rows, [_('User'), _('Action')])
-        print
+        print()
         printout(_("Available actions:"))
         actions = permsys.get_actions()
         actions.sort()
         text = ', '.join(actions)
         printout(wrap(text, initial_indent=' ', subsequent_indent=' ',
                       linesep='\n'))
-        print
+        print()
 
     def _do_add(self, user, *actions):
         permsys = PermissionSystem(self.env)
@@ -684,6 +703,8 @@ class PermissionAdmin(Component):
             except self.env.db_exc.IntegrityError:
                 printout(_("The user %(user)s already has permission "
                            "%(action)s.", user=user, action=action))
+            except TracError as e:
+                raise AdminCommandError(e)
 
     def _do_remove(self, user, *actions):
         permsys = PermissionSystem(self.env)
@@ -701,10 +722,12 @@ class PermissionAdmin(Component):
                             "%(user)s. The permission is granted through "
                             "a meta-permission or group.", action=action,
                             user=user)
-                else:
+                elif action in permsys.get_actions():
                     msg = _("Cannot remove permission %(action)s for user "
                             "%(user)s. The user has not been granted the "
                             "permission.", action=action, user=user)
+                else:
+                    msg = _("%(name)s is not a valid action.", name=action)
                 raise AdminCommandError(msg)
 
     def _do_export(self, filename=None):
@@ -718,7 +741,7 @@ class PermissionAdmin(Component):
                     actions = sorted(self.get_user_perms(user))
                     writer.writerow([s.encode(encoding, 'replace')
                                      for s in [user] + actions])
-        except IOError, e:
+        except IOError as e:
             raise AdminCommandError(
                 _("Cannot export to %(filename)s: %(error)s",
                   filename=path_to_unicode(filename or 'stdout'),
@@ -748,12 +771,12 @@ class PermissionAdmin(Component):
                     old_actions = self.get_user_perms(user)
                     for action in set(actions) - set(old_actions):
                         permsys.grant_permission(user, action)
-        except csv.Error, e:
+        except csv.Error as e:
             raise AdminCommandError(
                 _("Cannot import from %(filename)s line %(line)d: %(error)s ",
                   filename=path_to_unicode(filename or 'stdin'),
                   line=reader.line_num, error=e))
-        except IOError, e:
+        except IOError as e:
             raise AdminCommandError(
                 _("Cannot import from %(filename)s: %(error)s",
                   filename=path_to_unicode(filename or 'stdin'),
