@@ -1,4 +1,4 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
 # Copyright (C) 2003-2014 Edgewall Software
@@ -17,34 +17,36 @@
 # Author: Jonas Borgström <jonas@edgewall.com>
 #         Christopher Lenz <cmlenz@gmx.de>
 
-from __future__ import with_statement
-
+import abc
 import doctest
+import inspect
 import os
 import shutil
 import sys
+import types
 import unittest
 import StringIO
 
 try:
     from babel import Locale
-    locale_en = Locale.parse('en_US')
 except ImportError:
     locale_en = None
+else:
+    locale_en = Locale.parse('en_US')
 
 import trac.db.mysql_backend
 import trac.db.postgres_backend
 import trac.db.sqlite_backend
 from trac.config import Configuration
 from trac.core import ComponentManager, ComponentMeta, TracError
-from trac.db.api import DatabaseManager, _parse_db_str
+from trac.db.api import DatabaseManager, parse_connection_uri
 from trac.env import Environment
 from trac.perm import PermissionCache
 from trac.ticket.default_workflow import load_workflow_config_snippet
 from trac.util import translation
-from trac.util.datefmt import utc
-from trac.web.api import _RequestArgs, Request
-from trac.web.session import DetachedSession
+from trac.util.datefmt import time_now, utc
+from trac.web.api import _RequestArgs, Request, arg_list_to_args
+from trac.web.session import Session
 
 
 def Mock(bases=(), *initargs, **kw):
@@ -97,7 +99,25 @@ def Mock(bases=(), *initargs, **kw):
     """
     if not isinstance(bases, tuple):
         bases = (bases,)
-    cls = type('Mock', bases, {})
+
+    # if base classes have abstractmethod and abstractproperty,
+    # create dummy methods for abstracts
+    attrs = {}
+    def dummyfn(self, *args, **kwargs):
+        raise NotImplementedError
+    for base in bases:
+        if getattr(base, '__metaclass__', None) is not abc.ABCMeta:
+            continue
+        fn = types.UnboundMethodType(dummyfn, None, base)
+        for name, attr in inspect.getmembers(base):
+            if name in attrs:
+                continue
+            if isinstance(attr, abc.abstractproperty) or \
+                    isinstance(attr, types.UnboundMethodType) and \
+                    getattr(attr, '__isabstractmethod__', False) is True:
+                attrs[name] = fn
+
+    cls = type('Mock', bases, attrs)
     mock = cls(*initargs)
     for k, v in kw.items():
         setattr(mock, k, v)
@@ -139,8 +159,9 @@ def MockRequest(env, **kwargs):
     :keyword method: the HTTP request method
     :keyword path_info: the request path inside the application
 
-    Additionally `format`, `locale`, `lc_time` and `tz` can be
-    specified as keyword arguments.
+    Additionally `cookie`, `format`, `locale`, `lc_time`, `remote_addr`,
+    `remote_user`, `script_name`, `server_name`, `server_port`
+    and `tz` can be specified as keyword arguments.
 
     :since: 1.0.11
     """
@@ -152,20 +173,27 @@ def MockRequest(env, **kwargs):
     else:
         perm = PermissionCache(env, authname)
 
-    args = _RequestArgs()
-    args.update(kwargs.get('args', {}))
+    if 'arg_list' in kwargs:
+        arg_list = kwargs['arg_list']
+        args = arg_list_to_args(arg_list)
+    else:
+        args = _RequestArgs()
+        args.update(kwargs.get('args', {}))
+        arg_list = [(name, value) for name in args
+                                  for value in args.getlist(name)]
 
     environ = {
         'trac.base_url': env.abs_href(),
         'wsgi.url_scheme': 'http',
         'HTTP_ACCEPT_LANGUAGE': 'en-US',
+        'HTTP_COOKIE': kwargs.get('cookie', ''),
         'PATH_INFO': kwargs.get('path_info', '/'),
         'REQUEST_METHOD': kwargs.get('method', 'GET'),
-        'REMOTE_ADDR': '127.0.0.1',
-        'REMOTE_USER': authname,
-        'SCRIPT_NAME': '/trac.cgi',
-        'SERVER_NAME': 'localhost',
-        'SERVER_PORT': '80',
+        'REMOTE_ADDR': kwargs.get('remote_addr', '127.0.0.1'),
+        'REMOTE_USER': kwargs.get('remote_user', authname),
+        'SCRIPT_NAME': kwargs.get('script_name', '/trac.cgi'),
+        'SERVER_NAME': kwargs.get('server_name', 'localhost'),
+        'SERVER_PORT': kwargs.get('server_port', '80'),
     }
 
     status_sent = []
@@ -184,7 +212,7 @@ def MockRequest(env, **kwargs):
 
     from trac.web.chrome import Chrome
     req.callbacks.update({
-        'arg_list': None,
+        'arg_list': lambda req: arg_list,
         'args': lambda req: args,
         'authname': lambda req: authname,
         'chrome': Chrome(env).prepare_request,
@@ -194,7 +222,7 @@ def MockRequest(env, **kwargs):
         'locale': lambda req: kwargs.get('locale'),
         'incookie': Request._parse_cookies,
         'perm': lambda req: perm,
-        'session': lambda req: DetachedSession(env, authname),
+        'session': lambda req: Session(env, req),
         'tz': lambda req: kwargs.get('tz', utc),
         'use_xsendfile': False,
         'xsendfile_header': None,
@@ -251,7 +279,7 @@ class TestCaseSetup(unittest.TestCase):
 def get_dburi():
     dburi = os.environ.get('TRAC_TEST_DB_URI')
     if dburi:
-        scheme, db_prop = _parse_db_str(dburi)
+        scheme, db_prop = parse_connection_uri(dburi)
         # Assume the schema 'tractest' for PostgreSQL
         if scheme == 'postgres' and \
                 not db_prop.get('params', {}).get('schema'):
@@ -265,67 +293,36 @@ def get_dburi():
 
 
 def reset_sqlite_db(env, db_prop):
-    with env.db_transaction as db:
-        tables = db.get_table_names()
-        for table in tables:
-            db("DELETE FROM %s" % table)
-        return tables
+    """Deletes all data from the tables.
+
+    :since 1.1.3: deprecated and will be removed in 1.3.1. Use `reset_tables`
+                  from the database connection class instead.
+    """
+    return DatabaseManager(env).reset_tables()
 
 
 def reset_postgres_db(env, db_prop):
-    with env.db_transaction as db:
-        dbname = db.schema
-        if dbname:
-            # reset sequences
-            # information_schema.sequences view is available in
-            # PostgreSQL 8.2+ however Trac supports PostgreSQL 8.0+, uses
-            # pg_get_serial_sequence()
-            seqs = [seq for seq, in db("""
-                SELECT sequence_name
-                FROM (
-                    SELECT pg_get_serial_sequence(
-                        quote_ident(table_schema) || '.' ||
-                        quote_ident(table_name), column_name) AS sequence_name
-                    FROM information_schema.columns
-                    WHERE table_schema=%s) AS tab
-                WHERE sequence_name IS NOT NULL""", (dbname,))]
-            for seq in seqs:
-                db("ALTER SEQUENCE %s RESTART WITH 1" % seq)
-            # clear tables
-            tables = db.get_table_names()
-            for table in tables:
-                db("DELETE FROM %s" % db.quote(table))
-            # PostgreSQL supports TRUNCATE TABLE as well
-            # (see http://www.postgresql.org/docs/8.1/static/sql-truncate.html)
-            # but on the small tables used here, DELETE is actually much faster
-            return tables
+    """Deletes all data from the tables and resets autoincrement indexes.
+
+    :since 1.1.3: deprecated and will be removed in 1.3.1. Use `reset_tables`
+                  from the database connection class instead.
+    """
+    return DatabaseManager(env).reset_tables()
 
 
 def reset_mysql_db(env, db_prop):
-    dbname = os.path.basename(db_prop['path'])
-    if dbname:
-        with env.db_transaction as db:
-            tables = db("""SELECT table_name, auto_increment
-                           FROM information_schema.tables
-                           WHERE table_schema=%s""", (dbname,))
-            for table, auto_increment in tables:
-                if auto_increment is None or auto_increment == 1:
-                    # DELETE FROM is preferred to TRUNCATE TABLE, as the
-                    # auto_increment is not used or it is 1.
-                    db("DELETE FROM %s" % table)
-                else:
-                    # TRUNCATE TABLE is preferred to DELETE FROM, as we
-                    # need to reset the auto_increment in MySQL.
-                    db("TRUNCATE TABLE %s" % table)
-            return tables
+    """Deletes all data from the tables and resets autoincrement indexes.
 
+    :since 1.1.3: deprecated and will be removed in 1.3.1. Use `reset_tables`
+                  from the database connection class instead.
+    """
+    return DatabaseManager(env).reset_tables()
 
-# -- Environment stub
 
 class EnvironmentStub(Environment):
-    """A stub of the trac.env.Environment object for testing."""
+    """A stub of the trac.env.Environment class for testing."""
 
-    global_databasemanager = None
+    global_databasemanager = None  # Deprecated, will be removed in 1.3.1
     required = False
     abstract = True
 
@@ -383,30 +380,25 @@ class EnvironmentStub(Environment):
         for name_or_class in disable or ():
             config_key = self._component_name(name_or_class)
             self.config.set('components', config_key, 'disabled')
+        self.config.set('trac', 'permission_policies',
+                        'DefaultPermissionPolicy, LegacyAttachmentPolicy')
 
         # -- logging
         from trac.log import logger_handler_factory
         self.log, self._log_handler = logger_handler_factory('test')
 
         # -- database
-        self.config.set('components', 'trac.db.*', 'enabled')
         self.dburi = get_dburi()
+        self.config.set('components', 'trac.db.*', 'enabled')
+        self.config.set('trac', 'database', self.dburi)
+        self.config.set('trac', 'debug_sql', True)
+        self.global_databasemanager = DatabaseManager(self)  # Remove in 1.3.1
 
-        init_global = False
-        if self.global_databasemanager:
-            self.components[DatabaseManager] = self.global_databasemanager
-        else:
-            self.config.set('trac', 'database', self.dburi)
-            self.global_databasemanager = DatabaseManager(self)
-            self.config.set('trac', 'debug_sql', True)
-            init_global = not destroying
-
-        if default_data or init_global:
+        if not destroying:
             self.reset_db(default_data)
 
         self.config.set('trac', 'base_url', 'http://example.org/trac.cgi')
 
-        self.known_users = []
         translation.activate(locale_en)
 
     def reset_db(self, default_data=None):
@@ -416,70 +408,45 @@ class EnvironmentStub(Environment):
         :return: True upon success
         """
         from trac import db_default
-        scheme, db_prop = _parse_db_str(self.dburi)
         tables = []
-        remove_sqlite_db = False
+        dbm = DatabaseManager(self)
         try:
             with self.db_transaction as db:
                 db.rollback()  # make sure there's no transaction in progress
                 # check the database version
-                database_version = self.get_version()
-        except Exception:
-            # "Database not found ...",
-            # "OperationalError: no such table: system" or the like
+                db_version = dbm.get_database_version()
+        except (TracError, self.db_exc.DatabaseError):
             pass
         else:
-            if database_version == db_default.db_version:
+            if db_version == db_default.db_version:
                 # same version, simply clear the tables (faster)
-                m = sys.modules[__name__]
-                reset_fn = 'reset_%s_db' % scheme
-                if hasattr(m, reset_fn):
-                    tables = getattr(m, reset_fn)(self, db_prop)
+                tables = dbm.reset_tables()
             else:
                 # different version or version unknown, drop the tables
-                remove_sqlite_db = True
-                self.destroy_db(scheme, db_prop)
-
-        if scheme == 'sqlite' and remove_sqlite_db:
-            path = db_prop['path']
-            if path != ':memory:':
-                if not os.path.isabs(path):
-                    path = os.path.join(self.path, path)
-                self.global_databasemanager.shutdown()
-                os.remove(path)
+                self.destroy_db()
 
         if not tables:
-            self.global_databasemanager.init_db()
+            dbm.init_db()
             # we need to make sure the next get_db_cnx() will re-create
             # a new connection aware of the new data model - see #8518.
             if self.dburi != 'sqlite::memory:':
-                self.global_databasemanager.shutdown()
+                dbm.shutdown()
 
-        with self.db_transaction as db:
-            if default_data:
-                for table, cols, vals in db_default.get_data(db):
-                    db.executemany("INSERT INTO %s (%s) VALUES (%s)"
-                                   % (table, ','.join(cols),
-                                      ','.join(['%s'] * len(cols))), vals)
-            else:
-                db("INSERT INTO system (name, value) VALUES (%s, %s)",
-                   ('database_version', str(db_default.db_version)))
+        if default_data:
+            dbm.insert_into_tables(db_default.get_data)
+        else:
+            dbm.set_database_version(db_default.db_version)
 
     def destroy_db(self, scheme=None, db_prop=None):
-        if not (scheme and db_prop):
-            scheme, db_prop = _parse_db_str(self.dburi)
+        """Destroy the database.
+
+        :since 1.1.5: the `scheme` and `db_prop` parameters are deprecated and
+                      will be removed in 1.3.1.
+        """
         try:
-            with self.db_transaction as db:
-                if scheme == 'postgres' and db.schema:
-                    db('DROP SCHEMA %s CASCADE' % db.quote(db.schema))
-                elif scheme == 'mysql':
-                    for table in db.get_table_names():
-                        db("DROP TABLE IF EXISTS `%s`" % table)
-        except Exception:
-            # "TracError: Database not found...",
-            # psycopg2.ProgrammingError: schema "tractest" does not exist
+            DatabaseManager(self).destroy_db()
+        except (TracError, self.db_exc.DatabaseError):
             pass
-        return False
 
     def clear_component_registry(self):
         """Clear the component registry.
@@ -515,12 +482,38 @@ class EnvironmentStub(Environment):
         The database is reset, then the connections are shut down, and
         finally all environment files are removed from the disk.
         """
-        self.env.reset_db()
-        self.env.shutdown() # really closes the db connections
-        shutil.rmtree(self.env.path)
+        self.reset_db()
+        self.shutdown() # really closes the db connections
+        shutil.rmtree(self.path)
         if self._old_registry is not None:
             self.restore_component_registry()
 
+    # other utilities
+
+    def insert_users(self, users):
+        """Insert a tuple representing a user session to the
+        `session` and `session_attributes` tables.
+
+        The tuple can be length 3 with entries username, name and
+        email, in which case an authenticated user is assumed. The
+        tuple can also be length 4, with the last entry specifying
+        `1` for an authenticated user or `0` for an unauthenticated
+        user.
+        """
+        with self.db_transaction as db:
+            for row in users:
+                if len(row) == 3:
+                    username, name, email = row
+                    authenticated = 1
+                else:  # len(row) == 4
+                    username, name, email, authenticated = row
+                db("INSERT INTO session VALUES (%s, %s, %s)",
+                   (username, authenticated, int(time_now())))
+                db("INSERT INTO session_attribute VALUES (%s,%s,'name',%s)",
+                   (username, authenticated, name))
+                db("INSERT INTO session_attribute VALUES (%s,%s,'email',%s)",
+                   (username, authenticated, email))
+
     # overridden
 
     def is_component_enabled(self, cls):
@@ -528,9 +521,6 @@ class EnvironmentStub(Environment):
             return True
         return Environment.is_component_enabled(self, cls)
 
-    def get_known_users(self, cnx=None):
-        return self.known_users
-
 
 def locate(fn):
     """Locates a binary on the path.
@@ -549,20 +539,21 @@ def locate(fn):
 INCLUDE_FUNCTIONAL_TESTS = True
 
 
-def suite():
+def test_suite():
     import trac.tests
     import trac.admin.tests
     import trac.db.tests
     import trac.mimeview.tests
+    import trac.notification.tests
     import trac.search.tests
-    import trac.timeline.tests
     import trac.ticket.tests
+    import trac.timeline.tests
+    import trac.upgrades.tests
     import trac.util.tests
     import trac.versioncontrol.tests
     import trac.versioncontrol.web_ui.tests
     import trac.web.tests
     import trac.wiki.tests
-    import tracopt.mimeview.tests
     import tracopt.perm.tests
     import tracopt.ticket.tests
     import tracopt.versioncontrol.git.tests
@@ -570,22 +561,23 @@ def suite():
 
     suite = unittest.TestSuite()
     suite.addTest(trac.tests.basicSuite())
-    suite.addTest(trac.admin.tests.suite())
-    suite.addTest(trac.db.tests.suite())
-    suite.addTest(trac.mimeview.tests.suite())
-    suite.addTest(trac.search.tests.suite())
-    suite.addTest(trac.ticket.tests.suite())
-    suite.addTest(trac.timeline.tests.suite())
-    suite.addTest(trac.util.tests.suite())
-    suite.addTest(trac.versioncontrol.tests.suite())
-    suite.addTest(trac.versioncontrol.web_ui.tests.suite())
-    suite.addTest(trac.web.tests.suite())
-    suite.addTest(trac.wiki.tests.suite())
-    suite.addTest(tracopt.mimeview.tests.suite())
-    suite.addTest(tracopt.perm.tests.suite())
-    suite.addTest(tracopt.ticket.tests.suite())
-    suite.addTest(tracopt.versioncontrol.git.tests.suite())
-    suite.addTest(tracopt.versioncontrol.svn.tests.suite())
+    suite.addTest(trac.admin.tests.test_suite())
+    suite.addTest(trac.db.tests.test_suite())
+    suite.addTest(trac.mimeview.tests.test_suite())
+    suite.addTest(trac.notification.tests.test_suite())
+    suite.addTest(trac.search.tests.test_suite())
+    suite.addTest(trac.ticket.tests.test_suite())
+    suite.addTest(trac.timeline.tests.test_suite())
+    suite.addTest(trac.upgrades.tests.test_suite())
+    suite.addTest(trac.util.tests.test_suite())
+    suite.addTest(trac.versioncontrol.tests.test_suite())
+    suite.addTest(trac.versioncontrol.web_ui.tests.test_suite())
+    suite.addTest(trac.web.tests.test_suite())
+    suite.addTest(trac.wiki.tests.test_suite())
+    suite.addTest(tracopt.perm.tests.test_suite())
+    suite.addTest(tracopt.ticket.tests.test_suite())
+    suite.addTest(tracopt.versioncontrol.git.tests.test_suite())
+    suite.addTest(tracopt.versioncontrol.svn.tests.test_suite())
     suite.addTest(doctest.DocTestSuite(sys.modules[__name__]))
     if INCLUDE_FUNCTIONAL_TESTS:
         suite.addTest(trac.tests.functionalSuite())
@@ -597,4 +589,4 @@ if __name__ == '__main__':
     if '--skip-functional-tests' in sys.argv:
         sys.argv.remove('--skip-functional-tests')
         INCLUDE_FUNCTIONAL_TESTS = False
-    unittest.main(defaultTest='suite')
+    unittest.main(defaultTest='test_suite')
