@@ -25,7 +25,7 @@ import hashlib
 import inspect
 from itertools import izip, tee
 import locale
-import os.path
+import os
 from pkg_resources import find_distributions
 import random
 import re
@@ -410,7 +410,6 @@ def terminate(process):
         ctypes.windll.kernel32.CloseHandle(handle)
 
     def terminate_nix():
-        import os
         import signal
         try:
             os.kill(pid, signal.SIGTERM)
@@ -494,8 +493,12 @@ def is_path_below(path, parent):
     """Return True iff `path` is equal to parent or is located below `parent`
     at any level.
     """
-    path = os.path.abspath(path)
-    parent = os.path.abspath(parent)
+    def normalize(path):
+        if os.name == 'nt' and not isinstance(path, unicode):
+            path = path.decode('mbcs')
+        return os.path.normcase(os.path.abspath(path))
+    path = normalize(path)
+    parent = normalize(parent)
     return path == parent or path.startswith(parent + os.sep)
 
 
