@@ -21,9 +21,8 @@ from trac.config import ListOption
 from trac.core import *
 from trac.perm import IPermissionRequestor
 from trac.util import as_bool, is_path_below
-from trac.util.compat import any
 from trac.util.text import breakable_path, normalize_whitespace, print_table, \
-                           printout
+                           printerr, printout
 from trac.util.translation import _, ngettext, tag_
 from trac.versioncontrol import DbRepositoryProvider, InvalidRepository, \
                                 NoSuchChangeset, RepositoryManager, is_default
@@ -89,21 +88,23 @@ class VersionControlAdmin(Component):
         if len(args) == 1:
             return self.get_reponames()
 
-    def _do_changeset_added(self, reponame, *revs):
+    def _do_changeset_added(self, reponame, first_rev, *revs):
         if is_default(reponame):
             reponame = ''
         rm = RepositoryManager(self.env)
-        errors = rm.notify('changeset_added', reponame, revs)
+        errors = rm.notify('changeset_added', reponame, (first_rev,) + revs)
         for error in errors:
-            printout(error)
+            printerr(error)
+        return 2 if errors else 0
 
-    def _do_changeset_modified(self, reponame, *revs):
+    def _do_changeset_modified(self, reponame, first_rev, *revs):
         if is_default(reponame):
             reponame = ''
         rm = RepositoryManager(self.env)
-        errors = rm.notify('changeset_modified', reponame, revs)
+        errors = rm.notify('changeset_modified', reponame, (first_rev,) + revs)
         for error in errors:
-            printout(error)
+            printerr(error)
+        return 2 if errors else 0
 
     def _do_list(self):
         rm = RepositoryManager(self.env)
@@ -180,7 +181,9 @@ class RepositoryAdminPanel(Component):
     # IAdminPanelProvider methods
 
     def get_admin_panels(self, req):
-        if 'VERSIONCONTROL_ADMIN' in req.perm('admin', 'versioncontrol/repository'):
+        types = RepositoryManager(self.env).get_supported_types()
+        if types and 'VERSIONCONTROL_ADMIN' \
+                      in req.perm('admin', 'versioncontrol/repository'):
             yield ('versioncontrol', _('Version Control'), 'repository',
                    _('Repositories'))
 
@@ -207,7 +210,8 @@ class RepositoryAdminPanel(Component):
                     valid = True
                     for field in db_provider.repository_attrs:
                         value = normalize_whitespace(req.args.get(field))
-                        if (value is not None or field == 'hidden') \
+                        if (value is not None
+                            or field in ('hidden', 'sync_per_request')) \
                                 and value != info.get(field):
                             changes[field] = value
                     if 'dir' in changes and not \
@@ -217,9 +221,9 @@ class RepositoryAdminPanel(Component):
                         db_provider.modify_repository(reponame, changes)
                         add_notice(req, _('Your changes have been saved.'))
                         name = req.args.get('name')
-                        resync = tag.tt('trac-admin %s repository resync '
-                                        '"%s"' % (self.env.path,
-                                                  name or '(default)'))
+                        resync = tag.code('trac-admin "%s" repository resync '
+                                          '"%s"' % (self.env.path,
+                                                    name or '(default)'))
                         if 'dir' in changes:
                             msg = tag_('You should now run %(resync)s to '
                                        'synchronize Trac with the repository.',
@@ -231,10 +235,10 @@ class RepositoryAdminPanel(Component):
                                        resync=resync)
                             add_notice(req, msg)
                         if name and name != path_info and not 'alias' in info:
-                            cset_added = tag.tt('trac-admin %s changeset '
-                                                'added "%s" $REV'
-                                                % (self.env.path,
-                                                   name or '(default)'))
+                            cset_added = tag.code('trac-admin "%s" changeset '
+                                                  'added "%s" $REV'
+                                                  % (self.env.path,
+                                                     name or '(default)'))
                             msg = tag_('You will need to update your '
                                        'post-commit hook to call '
                                        '%(cset_added)s with the new '
@@ -269,15 +273,15 @@ class RepositoryAdminPanel(Component):
                         name = name or '(default)'
                         add_notice(req, _('The repository "%(name)s" has been '
                                           'added.', name=name))
-                        resync = tag.tt('trac-admin %s repository resync '
-                                        '"%s"' % (self.env.path, name))
+                        resync = tag.code('trac-admin "%s" repository resync '
+                                           '"%s"' % (self.env.path, name))
                         msg = tag_('You should now run %(resync)s to '
                                    'synchronize Trac with the repository.',
                                    resync=resync)
                         add_notice(req, msg)
-                        cset_added = tag.tt('trac-admin %s changeset '
-                                            'added "%s" $REV'
-                                            % (self.env.path, name))
+                        cset_added = tag.code('trac-admin "%s" changeset '
+                                              'added "%s" $REV'
+                                              % (self.env.path, name))
                         doc = tag.a(_("documentation"),
                                     href=req.href.wiki('TracRepositoryAdmin')
                                          + '#Synchronization')
@@ -333,7 +337,8 @@ class RepositoryAdminPanel(Component):
                                                          reponame in db_repos))
                             for (reponame, info) in all_repos.iteritems())
         types = sorted([''] + rm.get_supported_types())
-        data.update({'types': types, 'default_type': rm.repository_type,
+        data.update({'types': types,
+                     'default_type': rm.default_repository_type,
                      'repositories': repositories})
 
         return 'admin_repositories.html', data
@@ -342,13 +347,14 @@ class RepositoryAdminPanel(Component):
         """Extend repository info for rendering."""
         info['name'] = reponame
         info['hidden'] = as_bool(info.get('hidden'))
+        info['sync_per_request'] = as_bool(info.get('sync_per_request'))
         info['editable'] = editable
-        if not info.get('alias'):
+        if 'alias' not in info:
             if info.get('dir') is not None:
                 info['prettydir'] = breakable_path(info['dir']) or ''
             try:
                 repos = RepositoryManager(self.env).get_repository(reponame)
-            except InvalidRepository, e:
+            except InvalidRepository as e:
                 info['error'] = e
             except TracError:
                 pass  # Probably "unsupported connector"
