@@ -16,11 +16,14 @@
 # Author: Christopher Lenz <cmlenz@gmx.de>
 #         Matthew Good <trac@matt-good.net>
 
+from __future__ import print_function
+
 import cgi
 import dircache
 import fnmatch
 from functools import partial
 import gc
+import io
 import locale
 import os
 import pkg_resources
@@ -28,7 +31,6 @@ from pprint import pformat, pprint
 import re
 import sys
 import traceback
-import StringIO
 from urlparse import urlparse
 
 from genshi.builder import tag
@@ -36,15 +38,16 @@ from genshi.output import DocType
 from genshi.template import TemplateLoader
 
 from trac import __version__ as TRAC_VERSION
-from trac.config import BoolOption, ExtensionOption, Option, \
-                        OrderedExtensionsOption
+from trac.config import BoolOption, ChoiceOption, ConfigurationError, \
+                        ExtensionOption, Option, OrderedExtensionsOption
 from trac.core import *
 from trac.env import open_environment
 from trac.loader import get_plugin_info, match_plugins_to_frames
 from trac.perm import PermissionCache, PermissionError
 from trac.resource import ResourceNotFound
 from trac.util import arity, get_frame_info, get_last_traceback, hex_entropy, \
-                      read_file, safe_repr, translation, warn_setuptools_issue
+                      lazy, read_file, safe_repr, translation, \
+                      warn_setuptools_issue
 from trac.util.concurrency import threading
 from trac.util.datefmt import format_datetime, localtz, timezone, user_time
 from trac.util.text import exception_to_unicode, shorten_line, to_unicode, \
@@ -54,19 +57,23 @@ from trac.util.translation import _, get_negotiated_locale, has_babel, \
 from trac.web.api import HTTPBadRequest, HTTPException, HTTPForbidden, \
                          HTTPInternalError, HTTPNotFound, IAuthenticator, \
                          IRequestFilter, IRequestHandler, Request, \
-                         RequestDone, TracNotImplementedError
-from trac.web.chrome import Chrome, add_notice, add_warning
+                         RequestDone, TracNotImplementedError, \
+                         is_valid_default_handler
+from trac.web.chrome import Chrome, ITemplateProvider, add_notice, add_warning
 from trac.web.href import Href
-from trac.web.session import Session
+from trac.web.session import SessionDict, Session
 
 #: This URL is used for semi-automatic bug reports (see
 #: `send_internal_error`).  Please modify it to point to your own
 #: Trac instance if you distribute a patched version of Trac.
-default_tracker = 'http://trac.edgewall.org'
+default_tracker = 'https://trac.edgewall.org'
 
 
-class FakeSession(dict):
-    sid = None
+class FakeSession(SessionDict):
+
+    def get_session(self, sid, authenticated=False):
+        pass
+
     def save(self):
         pass
 
@@ -112,13 +119,14 @@ class RequestDispatcher(Component):
     """
     required = True
 
+    implements(ITemplateProvider)
+
     authenticators = ExtensionPoint(IAuthenticator)
     handlers = ExtensionPoint(IRequestHandler)
 
     filters = OrderedExtensionsOption('trac', 'request_filters',
                                       IRequestFilter,
-        doc="""Ordered list of filters to apply to all requests
-            (''since 0.10'').""")
+        doc="""Ordered list of filters to apply to all requests.""")
 
     default_handler = ExtensionOption('trac', 'default_handler',
                                       IRequestHandler, 'WikiModule',
@@ -127,7 +135,7 @@ class RequestDispatcher(Component):
 
         Options include `TimelineModule`, `RoadmapModule`,
         `BrowserModule`, `QueryModule`, `ReportModule`, `TicketModule`
-        and `WikiModule`. The default is `WikiModule`. (''since 0.9'')""")
+        and `WikiModule`.""")
 
     default_timezone = Option('trac', 'default_timezone', '',
         """The default timezone to use""")
@@ -137,7 +145,8 @@ class RequestDispatcher(Component):
         been set. (''since 0.12.1'')
         """)
 
-    default_date_format = Option('trac', 'default_date_format', '',
+    default_date_format = ChoiceOption('trac', 'default_date_format',
+                                       ('', 'iso8601'),
         """The date format. Valid options are 'iso8601' for selecting
         ISO 8601 format, or leave it empty which means the default
         date format will be inferred from the browser's default
@@ -161,7 +170,7 @@ class RequestDispatcher(Component):
         for authenticator in self.authenticators:
             try:
                 authname = authenticator.authenticate(req)
-            except TracError, e:
+            except TracError as e:
                 self.log.error("Can't authenticate using %s: %s",
                                authenticator.__class__.__name__,
                                exception_to_unicode(e, traceback=True))
@@ -182,10 +191,124 @@ class RequestDispatcher(Component):
         self.log.debug('Dispatching %r', req)
         chrome = Chrome(self.env)
 
-        # Setup request callbacks for lazily-evaluated properties
+        try:
+            # Select the component that should handle the request
+            chosen_handler = None
+            for handler in self._request_handlers.values():
+                if handler.match_request(req):
+                    chosen_handler = handler
+                    break
+            if not chosen_handler and req.path_info in ('', '/'):
+                chosen_handler = self._get_valid_default_handler(req)
+            # pre-process any incoming request, whether a handler
+            # was found or not
+            self.log.debug("Chosen handler is %s", chosen_handler)
+            chosen_handler = self._pre_process_request(req, chosen_handler)
+            if not chosen_handler:
+                if req.path_info.endswith('/'):
+                    # Strip trailing / and redirect
+                    target = unicode_quote(req.path_info.rstrip('/'))
+                    if req.query_string:
+                        target += '?' + req.query_string
+                    req.redirect(req.href + target, permanent=True)
+                raise HTTPNotFound('No handler matched request to %s',
+                                   req.path_info)
+
+            req.callbacks['chrome'] = partial(chrome.prepare_request,
+                                              handler=chosen_handler)
+
+            # Protect against CSRF attacks: we validate the form token
+            # for all POST requests with a content-type corresponding
+            # to form submissions
+            if req.method == 'POST':
+                ctype = req.get_header('Content-Type')
+                if ctype:
+                    ctype, options = cgi.parse_header(ctype)
+                if ctype in ('application/x-www-form-urlencoded',
+                             'multipart/form-data') and \
+                        req.args.get('__FORM_TOKEN') != req.form_token:
+                    if self.env.secure_cookies and req.scheme == 'http':
+                        msg = _('Secure cookies are enabled, you must '
+                                'use https to submit forms.')
+                    else:
+                        msg = _('Do you have cookies enabled?')
+                    raise HTTPBadRequest(_('Missing or invalid form token.'
+                                           ' %(msg)s', msg=msg))
+
+            # Process the request and render the template
+            resp = chosen_handler.process_request(req)
+            if resp:
+                if len(resp) == 2: # old Clearsilver template and HDF data
+                    self.log.error("Clearsilver template are no longer "
+                                   "supported (%s)", resp[0])
+                    raise TracError(
+                        _("Clearsilver templates are no longer supported, "
+                          "please contact your Trac administrator."))
+                # Genshi
+                template, data, content_type, method = \
+                    self._post_process_request(req, *resp)
+                if 'hdfdump' in req.args:
+                    req.perm.require('TRAC_ADMIN')
+                    # debugging helper - no need to render first
+                    out = io.BytesIO()
+                    pprint(data, out)
+                    req.send(out.getvalue(), 'text/plain')
+                self.log.debug("Rendering response from handler")
+                output = chrome.render_template(
+                        req, template, data, content_type, method=method,
+                        iterable=chrome.use_chunked_encoding)
+                req.send(output, content_type or 'text/html')
+            else:
+                self.log.debug("Empty or no response from handler. "
+                               "Entering post_process_request.")
+                self._post_process_request(req)
+        except RequestDone:
+            raise
+        except Exception as e:
+            # post-process the request in case of errors
+            err = sys.exc_info()
+            try:
+                self._post_process_request(req)
+            except RequestDone:
+                pass
+            except TracError as e2:
+                self.log.warning("Exception caught while post-processing"
+                                 " request: %s", exception_to_unicode(e2))
+            except Exception as e2:
+                if not (type(e) is type(e2) and e.args == e2.args):
+                    self.log.error("Exception caught while post-processing"
+                                   " request: %s",
+                                   exception_to_unicode(e2, traceback=True))
+            if isinstance(e, PermissionError):
+                raise HTTPForbidden(e)
+            if isinstance(e, ResourceNotFound):
+                raise HTTPNotFound(e)
+            if isinstance(e, NotImplementedError):
+                tb = traceback.extract_tb(err[2])[-1]
+                self.log.warning("%s caught from %s:%d in %s: %s",
+                                 e.__class__.__name__, tb[0], tb[1], tb[2],
+                                 to_unicode(e) or "(no message)")
+                raise HTTPInternalError(TracNotImplementedError(e))
+            if isinstance(e, TracError):
+                raise HTTPInternalError(e)
+            raise err[0], err[1], err[2]
+
+    # ITemplateProvider methods
+
+    def get_htdocs_dirs(self):
+        return []
+
+    def get_templates_dirs(self):
+        return [pkg_resources.resource_filename('trac.web', 'templates')]
+
+    # Internal methods
+
+    def set_default_callbacks(self, req):
+        """Setup request callbacks for lazily-evaluated properties.
+        """
         req.callbacks.update({
             'authname': self.authenticate,
-            'chrome': chrome.prepare_request,
+            'chrome': Chrome(self.env).prepare_request,
             'form_token': self._get_form_token,
             'lc_time': self._get_lc_time,
             'locale': self._get_locale,
@@ -196,108 +319,31 @@ class RequestDispatcher(Component):
             'xsendfile_header': self._get_xsendfile_header,
         })
 
-        try:
-            try:
-                # Select the component that should handle the request
-                chosen_handler = None
-                try:
-                    for handler in self.handlers:
-                        if handler.match_request(req):
-                            chosen_handler = handler
-                            break
-                    if not chosen_handler:
-                        if not req.path_info or req.path_info == '/':
-                            chosen_handler = self.default_handler
-                    # pre-process any incoming request, whether a handler
-                    # was found or not
-                    chosen_handler = \
-                        self._pre_process_request(req, chosen_handler)
-                except TracError, e:
-                    raise HTTPInternalError(e)
-                if not chosen_handler:
-                    if req.path_info.endswith('/'):
-                        # Strip trailing / and redirect
-                        target = unicode_quote(req.path_info.rstrip('/'))
-                        if req.query_string:
-                            target += '?' + req.query_string
-                        req.redirect(req.href + target, permanent=True)
-                    raise HTTPNotFound('No handler matched request to %s',
-                                       req.path_info)
+    @lazy
+    def _request_handlers(self):
+        return dict((handler.__class__.__name__, handler)
+                    for handler in self.handlers)
 
-                req.callbacks['chrome'] = partial(chrome.prepare_request,
-                                                  handler=chosen_handler)
+    def _get_valid_default_handler(self, req):
+        # Use default_handler from the Session if it is a valid value.
+        name = req.session.get('default_handler')
+        handler = self._request_handlers.get(name)
+        if handler and not is_valid_default_handler(handler):
+            handler = None
 
-                # Protect against CSRF attacks: we validate the form token
-                # for all POST requests with a content-type corresponding
-                # to form submissions
-                if req.method == 'POST':
-                    ctype = req.get_header('Content-Type')
-                    if ctype:
-                        ctype, options = cgi.parse_header(ctype)
-                    if ctype in ('application/x-www-form-urlencoded',
-                                 'multipart/form-data') and \
-                            req.args.get('__FORM_TOKEN') != req.form_token:
-                        if self.env.secure_cookies and req.scheme == 'http':
-                            msg = _('Secure cookies are enabled, you must '
-                                    'use https to submit forms.')
-                        else:
-                            msg = _('Do you have cookies enabled?')
-                        raise HTTPBadRequest(_('Missing or invalid form token.'
-                                               ' %(msg)s', msg=msg))
-
-                # Process the request and render the template
-                resp = chosen_handler.process_request(req)
-                if resp:
-                    if len(resp) == 2: # old Clearsilver template and HDF data
-                        self.log.error("Clearsilver template are no longer "
-                                       "supported (%s)", resp[0])
-                        raise TracError(
-                            _("Clearsilver templates are no longer supported, "
-                              "please contact your Trac administrator."))
-                    # Genshi
-                    template, data, content_type = \
-                              self._post_process_request(req, *resp)
-                    if 'hdfdump' in req.args:
-                        req.perm.require('TRAC_ADMIN')
-                        # debugging helper - no need to render first
-                        out = StringIO.StringIO()
-                        pprint(data, out)
-                        req.send(out.getvalue(), 'text/plain')
-
-                    output = chrome.render_template(
-                            req, template, data, content_type,
-                            iterable=chrome.use_chunked_encoding)
-                    req.send(output, content_type or 'text/html')
-                else:
-                    self._post_process_request(req)
-            except RequestDone:
-                raise
-            except:
-                # post-process the request in case of errors
-                err = sys.exc_info()
-                try:
-                    self._post_process_request(req)
-                except RequestDone:
-                    raise
-                except Exception, e:
-                    self.log.error("Exception caught while post-processing"
-                                   " request: %s",
-                                   exception_to_unicode(e, traceback=True))
-                raise err[0], err[1], err[2]
-        except PermissionError, e:
-            raise HTTPForbidden(e)
-        except ResourceNotFound, e:
-            raise HTTPNotFound(e)
-        except NotImplementedError, e:
-            tb = traceback.extract_tb(err[2])[-1]
-            self.log.warning("%s caught from %s:%d in %s: %s",
-                             e.__class__.__name__, tb[0], tb[1], tb[2],
-                             to_unicode(e) or "(no message)")
-            raise HTTPInternalError(TracNotImplementedError(e))
-        except TracError, e:
-            raise HTTPInternalError(e)
-
-    # Internal methods
+        if not handler:
+            # Use default_handler from project configuration.
+            handler = self.default_handler
+            if not is_valid_default_handler(handler):
+                raise ConfigurationError(
+                    tag_("%(handler)s is not a valid default handler. Please "
+                         "update %(option)s through the %(page)s page or by "
+                         "directly editing trac.ini.",
+                         handler=tag.code(handler.__class__.__name__),
+                         option=tag.code("[trac] default_handler"),
+                         page=tag.a(_("Basic Settings"),
+                                    href=req.href.admin('general/basics'))))
+        return handler
 
     def _get_perm(self, req):
         if isinstance(req.session, FakeSession):
@@ -308,15 +354,18 @@ class RequestDispatcher(Component):
     def _get_session(self, req):
         try:
             return Session(self.env, req)
-        except TracError, e:
-            self.log.error("can't retrieve session: %s",
-                           exception_to_unicode(e))
+        except TracError as e:
+            msg = "can't retrieve session: %s"
+            if isinstance(e, TracValueError):
+                self.log.warning(msg, e)
+            else:
+                self.log.error(msg, exception_to_unicode(e))
             return FakeSession()
 
     def _get_locale(self, req):
         if has_babel:
             preferred = req.session.get('language')
-            default = self.env.config.get('trac', 'default_language', '')
+            default = self.default_language
             negotiated = get_negotiated_locale([preferred, default] +
                                                req.languages)
             self.log.debug("Negotiated locale: %s -> %s", preferred,
@@ -356,8 +405,7 @@ class RequestDispatcher(Component):
             req.outcookie['trac_form_token']['path'] = req.base_path or '/'
             if self.env.secure_cookies:
                 req.outcookie['trac_form_token']['secure'] = True
-            if sys.version_info >= (2, 6):
-                req.outcookie['trac_form_token']['httponly'] = True
+            req.outcookie['trac_form_token']['httponly'] = True
             return req.outcookie['trac_form_token'].value
 
     def _get_use_xsendfile(self, req):
@@ -384,8 +432,12 @@ class RequestDispatcher(Component):
         return chosen_handler
 
     def _post_process_request(self, req, *args):
-        nbargs = len(args)
         resp = args
+        # `method` is optional in IRequestHandler's response. If not
+        # specified, the default value is appended to response.
+        if len(resp) == 3:
+            resp += (None,)
+        nbargs = len(resp)
         for f in reversed(self.filters):
             # As the arity of `post_process_request` has changed since
             # Trac 0.10, only filters with same arity gets passed real values.
@@ -394,6 +446,12 @@ class RequestDispatcher(Component):
             extra_arg_count = arity(f.post_process_request) - 1
             if extra_arg_count == nbargs:
                 resp = f.post_process_request(req, *resp)
+            elif extra_arg_count == nbargs - 1:
+                # IRequestFilters may modify the `method`, but the `method`
+                # is forwarded when not accepted by the IRequestFilter.
+                method = resp[-1]
+                resp = f.post_process_request(req, *resp[:-1])
+                resp += (method,)
             elif nbargs == 0:
                 f.post_process_request(req, *(None,)*extra_arg_count)
         return resp
@@ -414,6 +472,9 @@ def dispatch_request(environ, start_response):
         _warn_setuptools = True
         warn_setuptools_issue(out=environ.get('wsgi.errors'))
 
+    if sys.flags.optimize != 0:
+        raise EnvironmentError("Python with optimizations is not supported.")
+
     # SCRIPT_URL is an Apache var containing the URL before URL rewriting
     # has been applied, so we can use it to reconstruct logical SCRIPT_NAME
     script_url = environ.get('SCRIPT_URL')
@@ -519,9 +580,8 @@ def dispatch_request(environ, start_response):
                     'trac.web.version': mod_wsgi_version})
             env.webfrontend = environ.get('trac.web.frontend')
             if env.webfrontend:
-                env.systeminfo.append((env.webfrontend,
-                                       environ['trac.web.version']))
-    except Exception, e:
+                env.webfrontend_version = environ['trac.web.version']
+    except Exception as e:
         env_error = e
 
     req = RequestWithSession(environ, start_response)
@@ -557,13 +617,14 @@ def _dispatch_request(req, env, env_error):
     try:
         if not env and env_error:
             raise HTTPInternalError(env_error)
+        dispatcher = RequestDispatcher(env)
         try:
-            dispatcher = RequestDispatcher(env)
+            dispatcher.set_default_callbacks(req)
             dispatcher.dispatch(req)
-        except RequestDone, req_done:
+        except RequestDone as req_done:
             resp = req_done.iterable
         resp = resp or req._response or []
-    except HTTPException, e:
+    except HTTPException as e:
         _send_user_error(req, env, e)
     except Exception:
         send_internal_error(env, req, sys.exc_info())
@@ -573,7 +634,9 @@ def _dispatch_request(req, env, env_error):
 def _send_user_error(req, env, e):
     # See trac/web/api.py for the definition of HTTPException subclasses.
     if env:
-        env.log.warn('[%s] %s', req.remote_addr, exception_to_unicode(e))
+        env.log.warning('[%s] %s, %r, referrer %r',
+                        req.remote_addr, exception_to_unicode(e),
+                        req, req.environ.get('HTTP_REFERER'))
     data = {'title': e.title, 'type': 'TracError', 'message': e.message,
             'frames': [], 'traceback': None}
     if e.code == 403 and req.authname == 'anonymous':
@@ -589,13 +652,13 @@ def _send_user_error(req, env, e):
 
 def send_internal_error(env, req, exc_info):
     if env:
-        env.log.error("Internal Server Error: %r, referrer %r%s",
-                      req, req.environ.get('HTTP_REFERER'),
+        env.log.error("[%s] Internal Server Error: %r, referrer %r%s",
+                      req.remote_addr, req, req.environ.get('HTTP_REFERER'),
                       exception_to_unicode(exc_info[1], traceback=True))
     message = exception_to_unicode(exc_info[1])
     traceback = get_last_traceback()
 
-    frames, plugins, faulty_plugins = [], [], []
+    frames, plugins, faulty_plugins, interface_custom = [], [], [], []
     th = 'http://trac-hacks.org'
     has_admin = False
     try:
@@ -623,11 +686,12 @@ def send_internal_error(env, req, exc_info):
                 home_page = info.get('home_page', '')
                 if 'trac' in info:
                     tracker = info['trac']
-                elif urlparse(home_page)[1] == urlparse(th)[1]:
+                elif urlparse(home_page).netloc == urlparse(th).netloc:
                     tracker = th
                     plugin_name = info.get('home_page', '').rstrip('/') \
                                                            .split('/')[-1]
                     tracker_args = {'component': plugin_name}
+        interface_custom = Chrome(env).get_interface_customization_files()
 
     def get_description(_):
         if env and has_admin:
@@ -640,9 +704,15 @@ def send_internal_error(env, req, exc_info):
             enabled_plugins = "".join("|| '''`%s`''' || `%s` ||\n"
                                       % (p['name'], p['version'] or _('N/A'))
                                       for p in plugins)
+            files = Chrome(env).get_interface_customization_files().items()
+            interface_files = "".join("|| **%s** || %s ||\n"
+                                      % (k, ", ".join("`%s`" % f for f in v))
+                                      for k, v in sorted(files))
         else:
             sys_info = _("''System information not available''\n")
             enabled_plugins = _("''Plugin information not available''\n")
+            interface_files = _("''Interface customization information not "
+                                 "available''\n")
         return _("""\
 ==== How to Reproduce ====
 
@@ -661,12 +731,16 @@ User agent: `#USER_AGENT#`
 %(sys_info)s
 ==== Enabled Plugins ====
 %(enabled_plugins)s
+==== Interface Customization ====
+%(interface_customization)s
 ==== Python Traceback ====
 {{{
 %(traceback)s}}}""",
             method=req.method, path_info=req.path_info,
             req_args=pformat(req.args), sys_info=sys_info,
-            enabled_plugins=enabled_plugins, traceback=to_unicode(traceback))
+            enabled_plugins=enabled_plugins,
+            interface_customization=interface_files,
+            traceback=to_unicode(traceback))
 
     # Generate the description once in English, once in the current locale
     description_en = get_description(lambda s, **kw: safefmt(s, kw))
@@ -680,6 +754,7 @@ User agent: `#USER_AGENT#`
             'traceback': traceback, 'frames': frames,
             'shorten_line': shorten_line, 'repr': safe_repr,
             'plugins': plugins, 'faulty_plugins': faulty_plugins,
+            'interface': interface_custom,
             'tracker': tracker, 'tracker_args': tracker_args,
             'description': description, 'description_en': description_en}
 
@@ -722,7 +797,7 @@ def send_project_index(environ, start_response, parent_dir=None,
                     'description': env.project_description,
                     'href': href(env_name)
                 }
-            except Exception, e:
+            except Exception as e:
                 proj = {'name': env_name, 'description': to_unicode(e)}
             projects.append(proj)
         projects.sort(lambda x, y: cmp(x['name'].lower(), y['name'].lower()))
@@ -785,9 +860,9 @@ def get_environments(environ, warn=False):
         env_name = os.path.split(env_path)[1]
         if env_name in envs:
             if warn:
-                print >> sys.stderr, ('Warning: Ignoring project "%s" since '
-                                      'it conflicts with project "%s"'
-                                      % (env_path, envs[env_name]))
+                print('Warning: Ignoring project "%s" since it conflicts with'
+                      ' project "%s"' % (env_path, envs[env_name]),
+                      file=sys.stderr)
         else:
             envs[env_name] = env_path
     return envs
