@@ -22,7 +22,8 @@ import re
 
 from trac.admin.api import AdminCommandError, IAdminCommandProvider, \
                            console_date_format, get_console_locale
-from trac.core import Component, ExtensionPoint, TracError, implements
+from trac.core import Component, ExtensionPoint, TracError, TracValueError, \
+                      implements
 from trac.util import as_bool, as_int, hex_entropy, lazy
 from trac.util.datefmt import get_datetime_format_hint, format_date, \
                               parse_date, time_now, to_datetime, to_timestamp
@@ -43,8 +44,8 @@ COOKIE_KEY = 'trac_session'
 
 class SessionDict(dict):
 
-    def __init__(self):
-        super(SessionDict, self).__init__()
+    def __init__(self, *args, **kwargs):
+        super(SessionDict, self).__init__(*args, **kwargs)
         self.authenticated = False
         self.last_visit = 0
         self.sid = None
@@ -267,7 +268,7 @@ class Session(DetachedSession):
         refresh_cookie = False
 
         if not authenticated and not self._valid_sid_re.match(sid):
-            raise TracError(_("Session ID must be alphanumeric."))
+            raise TracValueError(_("Session ID must be alphanumeric."))
         if self.sid and sid != self.sid:
             refresh_cookie = True
 
@@ -286,8 +287,8 @@ class Session(DetachedSession):
         if new_sid == self.sid:
             return
         if not self._valid_sid_re.match(new_sid):
-            raise TracError(_("Session ID must be alphanumeric."),
-                            _("Error renaming session"))
+            raise TracValueError(_("Session ID must be alphanumeric."),
+                                 _("Error renaming session"))
         with self.env.db_transaction as db:
             if db("SELECT sid FROM session WHERE sid=%s", (new_sid,)):
                 raise TracError(_("Session '%(id)s' already exists. "
@@ -491,10 +492,10 @@ class SessionAdmin(Component):
             except Exception:
                 raise AdminCommandError(_("Session '%(sid)s' already exists",
                                           sid=sid))
-            if name is not None:
+            if name:
                 db("INSERT INTO session_attribute VALUES (%s,%s,'name',%s)",
                     (sid, authenticated, name))
-            if email is not None:
+            if email:
                 db("INSERT INTO session_attribute VALUES (%s,%s,'email',%s)",
                     (sid, authenticated, email))
         self.env.invalidate_known_users_cache()
@@ -518,8 +519,9 @@ class SessionAdmin(Component):
                 DELETE FROM session_attribute
                 WHERE sid=%s AND authenticated=%s AND name=%s
                 """, (sid, authenticated, attr))
-            db("INSERT INTO session_attribute VALUES (%s, %s, %s, %s)",
-               (sid, authenticated, attr, val))
+            if val:
+                db("INSERT INTO session_attribute VALUES (%s, %s, %s, %s)",
+                   (sid, authenticated, attr, val))
         self.env.invalidate_known_users_cache()
 
     def _do_delete(self, *sids):
@@ -555,3 +557,13 @@ class SessionAdmin(Component):
                       AND sid NOT IN (SELECT sid FROM session
                                       WHERE authenticated=0)
                 """)
+
+
+def get_session_attribute(env, sid, authenticated, name, default=None):
+    for row in env.db_query("""
+            SELECT value FROM session_attribute
+            WHERE sid=%s AND authenticated=%s AND name=%s
+            """, (sid, 1 if authenticated else 0, name)):
+        return row[0]
+    else:
+        return default
