@@ -26,8 +26,7 @@ from trac.core import *
 from trac.resource import IResourceManager
 from trac.util.text import unquote_label
 from trac.util.translation import _
-
-from .parser import WikiParser
+from trac.wiki.parser import WikiParser
 
 
 class IWikiChangeListener(Interface):
@@ -55,6 +54,9 @@ class IWikiChangeListener(Interface):
     def wiki_page_renamed(page, old_name):
         """Called when a page has been renamed."""
 
+    def wiki_page_comment_modified(page, old_comment):
+        """Called when a page comment has been modified."""
+
 
 class IWikiPageManipulator(Interface):
     """Components that need to do specific pre- and post- processing of
@@ -108,9 +110,6 @@ class IWikiMacroProvider(Interface):
            description.
         """
 
-    def render_macro(req, name, content):
-        """Return the HTML output of the macro :deprecated:"""
-
     def is_inline(content):
         """Return `True` if the content generated is an inline XHTML element.
 
@@ -121,10 +120,7 @@ class IWikiMacroProvider(Interface):
         """Called by the formatter when rendering the parsed wiki text.
 
         .. versionadded:: 0.11
-          This form is preferred over `render_macro`, as
-          you get the `formatter`, which knows the current `.context`
-          (and the `.req`, but ideally you shouldn't use it in your
-          macros).
+
         .. versionchanged:: 0.12
            added the `args` parameter
 
@@ -264,28 +260,30 @@ class WikiSystem(Component):
     macro_providers = ExtensionPoint(IWikiMacroProvider)
     syntax_providers = ExtensionPoint(IWikiSyntaxProvider)
 
+    realm = 'wiki'
+
     ignore_missing_pages = BoolOption('wiki', 'ignore_missing_pages', 'false',
-        """Enable/disable highlighting CamelCase links to missing pages
-        (''since 0.9'').""")
+        """Enable/disable highlighting CamelCase links to missing pages.
+        """)
 
     split_page_names = BoolOption('wiki', 'split_page_names', 'false',
-        """Enable/disable splitting the WikiPageNames with space characters
-        (''since 0.10'').""")
+        """Enable/disable splitting the WikiPageNames with space characters.
+        """)
 
     render_unsafe_content = BoolOption('wiki', 'render_unsafe_content', 'false',
         """Enable/disable the use of unsafe HTML tags such as `<script>` or
-        `<embed>` with the HTML [wiki:WikiProcessors WikiProcessor]
-        (''since 0.10.4'').
+        `<embed>` with the HTML [wiki:WikiProcessors WikiProcessor].
 
         For public sites where anonymous users can edit the wiki it is
-        recommended to leave this option disabled (which is the default).""")
+        recommended to leave this option disabled.
+        """)
 
     safe_schemes = ListOption('wiki', 'safe_schemes',
         'cvs, file, ftp, git, irc, http, https, news, sftp, smb, ssh, svn, '
         'svn+ssh',
         doc="""List of URI schemes considered "safe", that will be rendered as
         external links even if `[wiki] render_unsafe_content` is `false`.
-        (''since 0.11.8'')""")
+        """)
 
     @cached
     def pages(self):
@@ -309,6 +307,12 @@ class WikiSystem(Component):
         """Whether a page with the specified name exists."""
         return pagename.rstrip('/') in self.pages
 
+    def resolve_relative_name(self, pagename, referrer):
+        """Resolves a pagename relative to a referrer pagename."""
+        if pagename.startswith(('./', '../')) or pagename in ('.', '..'):
+            return self._resolve_relative_name(pagename, referrer)
+        return pagename
+
     # IWikiSyntaxProvider methods
 
     XML_NAME = r"[\w:](?<!\d)(?:[\w:.-]*[\w-])?"
@@ -406,7 +410,7 @@ class WikiSystem(Component):
             query = '&' + query[1:]
         pagename = pagename.rstrip('/') or 'WikiStart'
         referrer = ''
-        if formatter.resource and formatter.resource.realm == 'wiki':
+        if formatter.resource and formatter.resource.realm == self.realm:
             referrer = formatter.resource.id
         if pagename.startswith('/'):
             pagename = pagename.lstrip('/')
@@ -415,7 +419,7 @@ class WikiSystem(Component):
         else:
             pagename = self._resolve_scoped_name(pagename, referrer)
         label = unquote_label(label)
-        if 'WIKI_VIEW' in formatter.perm('wiki', pagename, version):
+        if 'WIKI_VIEW' in formatter.perm(self.realm, pagename, version):
             href = formatter.href.wiki(pagename, version=version) + query \
                    + fragment
             if self.has_page(pagename):
@@ -423,7 +427,8 @@ class WikiSystem(Component):
             else:
                 if ignore_missing:
                     return original_label or label
-                if 'WIKI_CREATE' in formatter.perm('wiki', pagename, version):
+                if 'WIKI_CREATE' in \
+                        formatter.perm(self.realm, pagename, version):
                     return tag.a(label + '?', class_='missing wiki',
                                  href=href, rel='nofollow')
                 else:
@@ -441,7 +446,7 @@ class WikiSystem(Component):
             if comp == '..':
                 if base:
                     base.pop()
-            elif comp and comp != '.':
+            elif comp != '.':
                 base.extend(components[i:])
                 break
         return '/'.join(base)
@@ -473,7 +478,7 @@ class WikiSystem(Component):
     # IResourceManager methods
 
     def get_resource_realms(self):
-        yield 'wiki'
+        yield self.realm
 
     def get_resource_description(self, resource, format, **kwargs):
         """
