@@ -24,17 +24,17 @@ import os
 from StringIO import StringIO
 
 from genshi.builder import tag, Element
-from genshi.core import Stream, Markup, escape
+from genshi.core import Stream
 from genshi.input import HTMLParser, ParseError
 from genshi.util import plaintext
 
 from trac.core import *
 from trac.mimeview import *
 from trac.resource import get_relative_resource, get_resource_url
-from trac.util import arity
+from trac.util import arity, as_int
+from trac.util.html import Markup, TracHTMLSanitizer, escape, to_fragment
 from trac.util.text import exception_to_unicode, shorten_line, to_unicode, \
-                           unicode_quote, unicode_quote_plus, unquote_label
-from trac.util.html import TracHTMLSanitizer
+                           unicode_quote, unquote_label
 from trac.util.translation import _, tag_
 from trac.wiki.api import WikiSystem, parse_args
 from trac.wiki.parser import WikiParser, parse_processor_args
@@ -131,7 +131,7 @@ class WikiProcessor(object):
         :param name: the name of the processor
         :param args: extra parameters for the processor
 
-        (since 0.11)
+        (''since 0.11'')
         """
         self.formatter = formatter
         self.env = formatter.env
@@ -171,7 +171,10 @@ class WikiProcessor(object):
                         if hasattr(macro_provider, 'expand_macro'):
                             self.processor = self._macro_processor
                         else:
-                            self.processor = self._legacy_macro_processor
+                            raise TracError(
+                                tag_("Pre-0.11 macros with the %(method)s "
+                                     "method are no longer supported.",
+                                     method=tag.code("render_macro")))
                         self.macro_provider = macro_provider
                         self.inline_check = getattr(macro_provider, 'is_inline',
                                                     False)
@@ -213,7 +216,12 @@ class WikiProcessor(object):
         return ''
 
     def _default_processor(self, text):
-        return tag.pre(text, class_="wiki")
+        if self.args and 'lineno' in self.args:
+            self.name = \
+                Mimeview(self.formatter.env).get_mimetype('text/plain')
+            return self._mimeview_processor(text)
+        else:
+            return tag.pre(text, class_="wiki")
 
     def _html_processor(self, text):
         if WikiSystem(self.env).render_unsafe_content:
@@ -221,7 +229,7 @@ class WikiProcessor(object):
         try:
             stream = Stream(HTMLParser(StringIO(text)))
             return (stream | self._sanitizer).render('xhtml', encoding=None)
-        except ParseError, e:
+        except ParseError as e:
             self.env.log.warn(e)
             line = unicode(text).splitlines()[e.lineno - 1].strip()
             return system_message(_('HTML parsing error: %(message)s',
@@ -275,7 +283,7 @@ class WikiProcessor(object):
             elt = self._elt_processor('tr', self._format_row, text)
             self.formatter.open_table()
             return elt
-        except ProcessorError, e:
+        except ProcessorError as e:
             return system_message(e)
 
     def _table_processor(self, text):
@@ -284,7 +292,7 @@ class WikiProcessor(object):
         self.args.setdefault('class', 'wiki')
         try:
             return self._elt_processor('table', self._format_table, text)
-        except ProcessorError, e:
+        except ProcessorError as e:
             return system_message(e)
 
     def _tablecell_processor(self, eltname, text):
@@ -332,12 +340,6 @@ class WikiProcessor(object):
 
     # generic processors
 
-    def _legacy_macro_processor(self, text): # TODO: remove in 0.12
-        self.env.log.warning('Executing pre-0.11 Wiki macro %s by '
-                             'provider %s', self.name, self.macro_provider)
-        return self.macro_provider.render_macro(self.formatter.req, self.name,
-                                                text)
-
     def _macro_processor(self, text):
         self.env.log.debug('Executing Wiki macro %s by provider %s',
                            self.name, self.macro_provider)
@@ -349,8 +351,23 @@ class WikiProcessor(object):
                                                     text)
 
     def _mimeview_processor(self, text):
-        return Mimeview(self.env).render(self.formatter.context,
-                                         self.name, text)
+        annotations = []
+        context = self.formatter.context.child()
+        args = self.args.copy() if self.args else self.args
+        if args and 'lineno' in args:
+            lineno = as_int(args.pop('lineno'), 1, min=1)
+            context.set_hints(lineno=lineno)
+            id = str(args.pop('id', '')) or \
+                 self.formatter._unique_anchor('a')
+            context.set_hints(id=id + '-')
+            if 'marks' in args:
+                context.set_hints(marks=args.pop('marks'))
+            annotations.append('lineno')
+        if args:  # Remaining args are assumed to be lexer options
+            context.set_hints(lexer_options=args)
+        return tag.div(class_='wiki-code')(
+            Mimeview(self.env).render(context, self.name, text,
+                                      annotations=annotations))
     # TODO: use convert('text/html') instead of render
 
     def process(self, text, in_paragraph=False):
@@ -407,13 +424,7 @@ class Formatter(object):
 
     flavor = 'default'
 
-    # 0.10 compatibility
-    INTERTRAC_SCHEME = WikiParser.INTERTRAC_SCHEME
-    QUOTED_STRING = WikiParser.QUOTED_STRING
-    LINK_SCHEME = WikiParser.LINK_SCHEME
-
     def __init__(self, env, context):
-        """Note: `req` is still temporarily used."""
         self.env = env
         self.context = context.child()
         self.context.set_hints(disable_warnings=True)
@@ -559,10 +570,10 @@ class Formatter(object):
         return self._indirect_tag_handler(match, 'MM_SUPERSCRIPT')
 
     def _inlinecode_formatter(self, match, fullmatch):
-        return tag.tt(fullmatch.group('inline'))
+        return tag.code(fullmatch.group('inline'))
 
     def _inlinecode2_formatter(self, match, fullmatch):
-        return tag.tt(fullmatch.group('inline2'))
+        return tag.code(fullmatch.group('inline2'))
 
     # pre-0.12 public API (no longer used by Trac itself but kept for plugins)
 
@@ -696,17 +707,7 @@ class Formatter(object):
             url = 'http://trac.edgewall.org'
         if url:
             name = intertrac.get(ns + '.title', 'Trac project %s' % ns)
-            compat = intertrac.getbool(ns + '.compat', 'false')
-            # set `compat` default to False now that 0.10 is widely used
-            # TODO: remove compatibility code completely for 1.0 release
-            if compat:
-                sep = target.find(':')
-                if sep != -1:
-                    url = '%s/%s/%s' % (url, target[:sep], target[sep + 1:])
-                else:
-                    url = '%s/search?q=%s' % (url, unicode_quote_plus(target))
-            else:
-                url = '%s/intertrac/%s' % (url, unicode_quote(target))
+            url = '%s/intertrac/%s' % (url, unicode_quote(target))
             if target:
                 title = _('%(target)s in %(name)s', target=target, name=name)
             else:
@@ -732,8 +733,7 @@ class Formatter(object):
                 return self._make_ext_link(url, label, title)
 
     def _make_ext_link(self, url, text, title=''):
-        local_url = self.env.project_url or \
-                    (self.req or self.env).abs_href.base
+        local_url = self.env.project_url or self.env.abs_href.base
         if not url.startswith(local_url):
             return tag.a(tag.span(u'\u200b', class_="icon"), text,
                          class_="ext-link", href=url, title=title or None)
@@ -753,6 +753,15 @@ class Formatter(object):
             label = format_to_oneliner(self.env, self.context, label)
         return '<span class="wikianchor" id="%s">%s</span>' % (anchor, label)
 
+    def _unique_anchor(self, anchor):
+        i = 1
+        anchor_base = anchor
+        while anchor in self._anchors:
+            anchor = anchor_base + str(i)
+            i += 1
+        self._anchors[anchor] = True
+        return anchor
+
     # WikiMacros or WikiCreole links
 
     def _macrolink_formatter(self, match, fullmatch):
@@ -791,13 +800,14 @@ class Formatter(object):
             args = fullmatch.group('macroargs')
         try:
             return macro.ensure_inline(macro.process(args))
-        except MacroError, e:
-            return system_message(e)
-        except Exception, e:
+        except MacroError as e:
+            return system_message(_("Macro %(name)s(%(args)s) failed",
+                                    name=name, args=args), to_fragment(e))
+        except Exception as e:
             self.env.log.error('Macro %s(%s) failed:%s', name, args,
                                exception_to_unicode(e, traceback=True))
             return system_message(_("Error: Macro %(name)s(%(args)s) failed",
-                                    name=name, args=args), to_unicode(e))
+                                    name=name, args=args), to_fragment(e))
 
     # Headings
 
@@ -819,12 +829,7 @@ class Formatter(object):
             if not anchor or anchor[0].isdigit() or anchor[0] in '.-':
                 # an ID must start with a Name-start character in XHTML
                 anchor = 'a' + anchor # keeping 'a' for backward compat
-        i = 1
-        anchor_base = anchor
-        while anchor in self._anchors:
-            anchor = anchor_base + str(i)
-            i += 1
-        self._anchors[anchor] = True
+        anchor = self._unique_anchor(anchor)
         if shorten:
             heading = format_to_oneliner(self.env, self.context, htext, True)
         return (depth, heading, anchor)
@@ -938,7 +943,11 @@ class Formatter(object):
     # Definition Lists
 
     def _definition_formatter(self, match, fullmatch):
-        tmp = '</dd>' if self.in_def_list else '<dl class="wiki">'
+        if self.in_def_list:
+            tmp = '</dd>'
+        else:
+            self.close_paragraph()
+            tmp = '<dl class="wiki">'
         definition = match[:match.find('::')]
         tmp += '<dt>%s</dt><dd>' % format_to_oneliner(self.env, self.context,
                                                       definition)
@@ -1186,7 +1195,7 @@ class Formatter(object):
     def _exec_processor(self, processor, text):
         try:
             return processor.process(text)
-        except Exception, e:
+        except Exception as e:
             self.env.log.error('Processor %s failed:%s', processor.name,
                                exception_to_unicode(e, traceback=True))
             return system_message(_("Error: Processor %(name)s failed",
@@ -1268,7 +1277,6 @@ class Formatter(object):
         self.paragraph_open = 0
         return source
 
-
     def format(self, text, out=None, escape_newlines=False):
         text = self.reset(text, out)
         if isinstance(text, basestring):
@@ -1316,6 +1324,10 @@ class Formatter(object):
             if not line.startswith(' '):
                 self._tabstops = []
 
+            # Handle end of indentation
+            if not line.startswith(' ') and self._quote_stack:
+                self.close_indentation()
+
             self.in_list_item = False
             self.in_quote = False
             # Throw a bunch of regexps on the problem
