@@ -14,10 +14,9 @@
 #
 # Author: Christopher Lenz <cmlenz@gmx.de>
 
-from __future__ import with_statement
-
 from fnmatch import fnmatchcase
 from itertools import groupby
+import fnmatch
 import inspect
 import os
 import re
@@ -41,10 +40,12 @@ from trac.web.chrome import chrome_resource_path
 from trac.wiki.api import IWikiMacroProvider, WikiSystem, parse_args
 from trac.wiki.formatter import (
     MacroError, OutlineFormatter, extract_link, format_to_html,
-    format_to_oneliner
+    format_to_oneliner, system_message
 )
+from trac.wiki.interwiki import InterWikiMap
 
 
+# TODO: should be moved in .api
 class WikiMacroBase(Component):
     """Abstract base class for wiki macros."""
 
@@ -89,11 +90,16 @@ class TitleIndexMacro(WikiMacroBase):
 
     Accepts a prefix string as parameter: if provided, only pages with names
     that start with the prefix are included in the resulting list. If this
-    parameter is omitted, all pages are listed.
-    If the prefix is specified, a second argument of value `hideprefix`
-    can be given as well, in order to remove that prefix from the output.
+    parameter is omitted, all pages are listed. If the prefix is specified,
+    a second argument of value `hideprefix` can be given as well, in order
+    to remove that prefix from the output.
 
-    Alternate `format` and `depth` named parameters can be specified:
+    The prefix string supports the standard relative-path notation ''when
+    using the macro in a wiki page''. A prefix string starting with `./`
+    will be relative to the current page, and parent pages can be
+    specified using `../`.
+
+    Several named parameters can be specified:
      - `format=compact`: The pages are displayed as comma-separated links.
      - `format=group`: The list of pages will be structured in groups
        according to common prefix. This format also supports a `min=n`
@@ -124,7 +130,6 @@ class TitleIndexMacro(WikiMacroBase):
         minsize = _arg_as_int(kw.get('min', 1), 'min', min=1)
         minsize_group = max(minsize, 2)
         depth = _arg_as_int(kw.get('depth', -1), 'depth', min=-1)
-        start = prefix.count('/') if prefix else 0
         format = kw.get('format', '')
 
         def parse_list(name):
@@ -134,13 +139,18 @@ class TitleIndexMacro(WikiMacroBase):
         includes = parse_list('include') or ['*']
         excludes = parse_list('exclude')
 
+        wiki = formatter.wiki
+        resource = formatter.resource
+        if prefix and resource and resource.realm == 'wiki':
+            prefix = wiki.resolve_relative_name(prefix, resource.id)
+
+        start = prefix.count('/') if prefix else 0
+
         if hideprefix:
             omitprefix = lambda page: page[len(prefix):]
         else:
             omitprefix = lambda page: page
 
-        wiki = formatter.wiki
-
         pages = sorted(page for page in wiki.get_pages(prefix)
                        if (depth < 0 or depth >= page.count('/') - start)
                        and 'WIKI_VIEW' in formatter.perm('wiki', page)
@@ -282,7 +292,7 @@ class TitleIndexMacro(WikiMacroBase):
             'group': lambda p: render_group(tree_group(split_pages_group(p))),
             'hierarchy': lambda p: render_hierarchy(
                                     tree_hierarchy(split_pages_hierarchy(p))),
-            }.get(format, None)
+            }.get(format)
 
         if transform:
             titleindex = transform(pages)
@@ -457,14 +467,15 @@ class ImageMacro(WikiMacroBase):
      * `file` to refer to a local attachment named 'file'. This only works from
        within that wiki page or a ticket.
 
-    Also, the file specification may refer to repository files, using the
-    `source:file` syntax (`source:file@rev` works also).
-
-    Files can also be accessed with a direct URLs; `/file` for a
-    project-relative, `//file` for a server-relative, or `http://server/file`
-    for absolute location of the file. The
-    [http://tools.ietf.org/html/rfc2397 rfc2397] `data` URL scheme is also
-    supported if the URL is enclosed in quotes.
+    The file specification may also refer to:
+     * repository files, using the `source:file` syntax
+       (`source:file@rev` works also).
+     * files, using direct URLs: `/file` for a project-relative,
+       `//file` for a server-relative, or `http://server/file` for
+       absolute location. An InterWiki prefix may be used.
+     * embedded data using the
+       [http://tools.ietf.org/html/rfc2397 rfc2397] `data` URL scheme,
+       provided the URL is enclosed in quotes.
 
     The remaining arguments are optional and allow configuring the attributes
     and style of the rendered `<img>` element:
@@ -527,13 +538,15 @@ class ImageMacro(WikiMacroBase):
     _quoted_re = re.compile("(?:[\"'])(.*)(?:[\"'])$")
 
     def expand_macro(self, formatter, name, content):
-        # args will be null if the macro is called without parenthesis.
-        if not content:
+        args = None
+        if content:
+            content = stripws(content)
+            # parse arguments
+            # we expect the 1st argument to be a filename (filespec)
+            args = [stripws(arg) for arg
+                                 in self._split_args_re.split(content)[1::2]]
+        if not args:
             return ''
-        # parse arguments
-        # we expect the 1st argument to be a filename (filespec)
-        args = [stripws(arg) for arg
-                             in self._split_args_re.split(content)[1::2]]
         # strip unicode white-spaces and ZWSPs are copied from attachments
         # section (#10668)
         filespec = args.pop(0)
@@ -601,11 +614,16 @@ class ImageMacro(WikiMacroBase):
         # parse filespec argument to get realm and id if contained.
         parts = [i.strip('''['"]''')
                  for i in self._split_filespec_re.split(filespec)[1::2]]
+        realm = parts[0] if parts else None
         url = raw_url = desc = None
         attachment = None
-        if parts and parts[0] in ('http', 'https', 'ftp', 'data'):  # absolute
+        interwikimap = InterWikiMap(self.env)
+        if realm in ('http', 'https', 'ftp', 'data'):  # absolute
             raw_url = url = filespec
             desc = url.rsplit('?')[0]
+        elif realm in interwikimap:
+            url, desc = interwikimap.url(realm, ':'.join(parts[1:]))
+            raw_url = url
         elif filespec.startswith('//'):       # server-relative
             raw_url = url = filespec[1:]
             desc = url.rsplit('?')[0]
@@ -659,19 +677,24 @@ class ImageMacro(WikiMacroBase):
         elif len(parts) == 1:  # it's an attachment of the current resource
             attachment = formatter.resource.child('attachment', filespec)
         else:
-            raise TracError(_("No filespec given"))
-        if attachment and 'ATTACHMENT_VIEW' in formatter.perm(attachment):
-            url = get_resource_url(self.env, attachment, formatter.href)
-            raw_url = get_resource_url(self.env, attachment, formatter.href,
-                                       format='raw')
+            return system_message(_("No filespec given"))
+        if attachment:
             try:
                 desc = get_resource_summary(self.env, attachment)
             except ResourceNotFound:
+                link = None
                 raw_url = chrome_resource_path(formatter.context.req,
                                                'common/attachment.png')
                 desc = _('No image "%(id)s" attached to %(parent)s',
                          id=attachment.id,
                          parent=get_resource_name(self.env, attachment.parent))
+            else:
+                if 'ATTACHMENT_VIEW' in formatter.perm(attachment):
+                    url = get_resource_url(self.env, attachment,
+                                           formatter.href)
+                    raw_url = get_resource_url(self.env, attachment,
+                                               formatter.href, format='raw')
+
         for key in ('title', 'alt'):
             if desc and key not in attr:
                 attr[key] = desc
@@ -714,7 +737,7 @@ class MacroListMacro(WikiMacroBase):
                     name_descriptions = [
                         (name, macro_provider.get_macro_description(name))
                         for name in names]
-                except Exception, e:
+                except Exception as e:
                     yield system_message(
                         _("Error: Can't get description for macro %(name)s",
                           name=names[0]), e), names
@@ -752,38 +775,56 @@ class TracIniMacro(WikiMacroBase):
     _description = cleandoc_(
     """Produce documentation for the Trac configuration file.
 
-    Typically, this will be used in the TracIni page.
-    Optional arguments are a configuration section filter,
+    Typically, this will be used in the TracIni page. The macro accepts
+    two ordered arguments and two named arguments.
+
+    The ordered arguments are a configuration section filter,
     and a configuration option name filter: only the configuration
     options whose section and name start with the filters are output.
+
+    The named arguments can be specified:
+
+     section :: a glob-style filtering on the section names
+     option  :: a glob-style filtering on the option names
     """)
 
     def expand_macro(self, formatter, name, content):
         from trac.config import ConfigSection, Option
-        section_filter = key_filter = ''
+
         args, kw = parse_args(content)
-        if args:
-            section_filter = args.pop(0).strip()
-        if args:
-            key_filter = args.pop(0).strip()
+        filters = {}
+        for name, index in (('section', 0), ('option', 1)):
+            pattern = kw.get(name, '').strip()
+            if pattern:
+                filters[name] = fnmatch.translate(pattern)
+                continue
+            prefix = args[index].strip() if index < len(args) else ''
+            if prefix:
+                filters[name] = re.escape(prefix)
+        has_option_filter = 'option' in filters
+        for name in ('section', 'option'):
+            filters[name] = re.compile(filters[name], re.IGNORECASE).match \
+                            if name in filters \
+                            else lambda v: True
+        section_filter = filters['section']
+        option_filter = filters['option']
 
-        def getdoc(option_or_section):
-            doc = to_unicode(option_or_section.__doc__)
-            if doc:
-                doc = dgettext(option_or_section.doc_domain, doc)
-            return doc
-
-        registry = ConfigSection.get_registry(self.compmgr)
-        sections = dict((name, getdoc(section))
-                        for name, section in registry.iteritems()
-                        if name.startswith(section_filter))
-
-        registry = Option.get_registry(self.compmgr)
+        section_registry = ConfigSection.get_registry(self.compmgr)
+        option_registry = Option.get_registry(self.compmgr)
         options = {}
-        for (section, key), option in registry.iteritems():
-            if section.startswith(section_filter):
+        for (section, key), option in option_registry.iteritems():
+            if section_filter(section) and option_filter(key):
                 options.setdefault(section, {})[key] = option
-                sections.setdefault(section, '')
+        if not has_option_filter:
+            for section in section_registry:
+                if section_filter(section):
+                    options.setdefault(section, {})
+        for section in options:
+            options[section] = sorted(options[section].itervalues(),
+                                      key=lambda option: option.name)
+        sections = [(section, section_registry[section].doc
+                              if section in section_registry else '')
+                    for section in sorted(options)]
 
         def default_cell(option):
             default = option.default
@@ -793,20 +834,27 @@ class TracIniMacro(WikiMacroBase):
             else:
                 return tag.td(_("(no default)"), class_='nodefault')
 
+        def options_table(section, options):
+            if options:
+                return tag.table(class_='wiki')(
+                    tag.tbody(
+                        tag.tr(
+                            tag.td(tag.a(tag.code(option.name),
+                                         class_='tracini-option',
+                                         href='#%s-%s-option' %
+                                              (section, option.name))),
+                            tag.td(format_to_html(self.env, formatter.context,
+                                                  option.doc)),
+                            default_cell(option),
+                            id='%s-%s-option' % (section, option.name),
+                            class_='odd' if idx % 2 else 'even')
+                     for idx, option in enumerate(options)))
+
         return tag.div(class_='tracini')(
             (tag.h3(tag.code('[%s]' % section), id='%s-section' % section),
              format_to_html(self.env, formatter.context, section_doc),
-             tag.table(class_='wiki')(tag.tbody(
-                 tag.tr(tag.td(tag.tt(option.name)),
-                        tag.td(format_to_html(self.env, formatter.context,
-                                              getdoc(option))),
-                        default_cell(option),
-                        class_='odd' if idx % 2 else 'even')
-                 for idx, option in
-                    enumerate(sorted(options.get(section, {}).itervalues(),
-                                     key=lambda o: o.name))
-                 if option.name.startswith(key_filter))))
-            for section, section_doc in sorted(sections.iteritems()))
+             options_table(section, options.get(section)))
+            for section, section_doc in sections)
 
 
 class KnownMimeTypesMacro(WikiMacroBase):
@@ -839,7 +887,7 @@ class KnownMimeTypesMacro(WikiMacroBase):
                                  href=formatter.context.href.wiki(
                                      'WikiProcessors'))))),
                 tag.tbody(
-                    tag.tr(tag.th(tag.tt(mime_type),
+                    tag.tr(tag.th(tag.code(mime_type),
                                   style="text-align: left"),
                            tag.td(tag.code(
                                ' '.join(sorted(mime_types[mime_type])))))
@@ -908,6 +956,6 @@ def _arg_as_int(val, key=None, min=None, max=None):
     int_val = as_int(val, None, min=min, max=max)
     if int_val is None:
         raise MacroError(tag_("Invalid macro argument %(expr)s",
-                              expr=tag.tt("%s=%s" % (key, val))
-                                   if key else tag.tt(val)))
+                              expr=tag.code("%s=%s" % (key, val))
+                                   if key else tag.code(val)))
     return int_val
