@@ -12,8 +12,6 @@
 # individuals. For the exact contribution history, see the revision
 # history and logs, available at http://trac.edgewall.org/log/.
 
-from __future__ import with_statement
-
 import os
 import codecs
 from collections import deque
@@ -25,6 +23,7 @@ from subprocess import Popen, PIPE
 from threading import Lock
 import weakref
 
+from trac.core import TracBaseError
 from trac.util import terminate
 from trac.util.compat import close_fds
 from trac.util.datefmt import time_now
@@ -33,7 +32,7 @@ from trac.util.text import to_unicode
 __all__ = ['GitError', 'GitErrorSha', 'Storage', 'StorageFactory']
 
 
-class GitError(Exception):
+class GitError(TracBaseError):
     pass
 
 class GitErrorSha(GitError):
@@ -142,7 +141,7 @@ class GitCore(object):
     def __execute(self, git_cmd, *cmd_args):
         """execute git command and return file-like object of stdout"""
 
-        #print >>sys.stderr, "DEBUG:", git_cmd, cmd_args
+        #print("DEBUG:", git_cmd, cmd_args, file=sys.stderr)
 
         p = self.__pipe(git_cmd, stdout=PIPE, stderr=PIPE, *cmd_args)
         stdout_data, stderr_data = p.communicate()
@@ -344,7 +343,7 @@ class Storage(object):
             result['v_compatible'] = split_version >= GIT_VERSION_MIN_REQUIRED
             return result
 
-        except Exception, e:
+        except Exception as e:
             raise GitError("Could not retrieve GIT version (tried to "
                            "execute/parse '%s --version' but got %s)"
                            % (git_bin, repr(e)))
@@ -415,7 +414,7 @@ class Storage(object):
         try:
             with open(head_file, 'rb'):
                 pass
-        except IOError, e:
+        except IOError as e:
             raise GitError("Make sure the Git repository '%s' is readable: %s"
                            % (git_dir, to_unicode(e)))
 
@@ -792,27 +791,18 @@ class Storage(object):
                       for name, rev_ in self.rev_cache.iter_tags()
                       if rev is None or rev == rev_)
 
-    def ls_tree(self, rev, path=''):
+    def ls_tree(self, rev, path='', recursive=False):
         rev = rev and str(rev) or 'HEAD' # paranoia
-
-        path = self._fs_from_unicode(path)
-
-        if path.startswith('/'):
-            path = path[1:]
-
-        tree = self.repo.ls_tree('-z', '-l', rev, '--', path).split('\0')
+        path = self._fs_from_unicode(path).lstrip('/') or '.'
+        tree = self.repo.ls_tree('-zlr' if recursive else '-zl',
+                                 rev, '--', path).split('\0')
 
         def split_ls_tree_line(l):
             """split according to '<mode> <type> <sha> <size>\t<fname>'"""
 
             meta, fname = l.split('\t', 1)
             _mode, _type, _sha, _size = meta.split()
-
-            if _size == '-':
-                _size = None
-            else:
-                _size = int(_size)
-
+            _size = None if _size == '-' else int(_size)
             return _mode, _type, _sha, _size, self._fs_to_unicode(fname)
 
         return [ split_ls_tree_line(e) for e in tree if e ]
@@ -908,10 +898,10 @@ class Storage(object):
         p = []
         change = {}
         next_path = []
-        base_path = self._fs_from_unicode(base_path)
+        base_path = self._fs_from_unicode(base_path) or '.'
 
         def name_status_gen():
-            p[:] = [self.repo.log_pipe('--pretty=format:%n%H',
+            p[:] = [self.repo.log_pipe('--pretty=format:%n%H', '--no-renames',
                                        '--name-status', sha, '--', base_path)]
             f = p[0].stdout
             for l in f:
@@ -1015,7 +1005,7 @@ class Storage(object):
         # diff-tree returns records with the following structure:
         # :<old-mode> <new-mode> <old-sha> <new-sha> <change> NUL <old-path> NUL [ <new-path> NUL ]
 
-        path = self._fs_from_unicode(path).strip('/')
+        path = self._fs_from_unicode(path).strip('/') or '.'
         diff_tree_args = ['-z', '-r']
         if find_renames:
             diff_tree_args.append('-M')
